class rRundata;
class GridInfo;
typedef rRundata * LPRRDATA;

enum ShapeType {
	TSHAPE_RECT = 0,
	TSHAPE_ELLIPSE = 1,
	TSHAPE_POLY = 2
};

enum FillType {
	TFILL_FLAT = 0,
	TFILL_GRAD = 1,
	TFILL_PATTERN = 2,
	TFILL_IMAGE = 3
};

enum GradDirection {
	GRAD_HORIZ = 0,
	GRAD_VERT = 1
};

enum ImageOptions {
	IMGOPT_RESAMPLE = 0x01,
	IMGOPT_RESAMPLE_EDGE = 0x02
};

// --------------------------------
// RUNNING OBJECT DATA STRUCTURE
// --------------------------------
// If you want to store anything between actions/conditions/expressions
// you should store it here

typedef struct tagRDATA
{
	#include "MagicRDATA.h"
	
	int				width;
	int				height;

	int				shape;
	int				fillType;
	int				gradDir;

	COLORREF		fillColor;
	COLORREF		fillColor2;
	COLORREF		lineColor;
	int				lineWeight;

	bool			useTrans;
	DWORD			imageOptions;

	COLORREF		transColor;
	WORD*			patterns;
	WORD*			images;

	size_t			patternCount;
	size_t			imageCount;

	char*			text;
	COLORREF		textColor;
	HFONT			textFont;
	DWORD			textAlignFlags;
	DWORD			textEffect;
	DWORD			textCoef;

	int				polyn;
	char*			polyCoords;
	tagPOINT*		polySet;

	int				rebuild;
	int				rebuildPoly;
	int				rebuildTrans;

	cSurface*		pSf;
	sMask*			pMask;

	int				hotX;
	int				hotY;

	// Transformed data
	cSurface*		transSf;
	sMask*			transMask;

	int				diffWidth;
	int				diffHeight;
	
} RUNDATA;
typedef	RUNDATA	* LPRDATA;

// --------------------------------
// EDITION OF OBJECT DATA STRUCTURE
// --------------------------------
// These values let you store data in your extension that will be saved in the MFA
// You should use these with properties

struct tagEDATA_V1
{
	extHeader		eHeader;
	int				width;
	int				height;

	int				shape;
	int				fillType;
	int				gradDir;

	COLORREF		fillColor;
	COLORREF		fillColor2;
	COLORREF		lineColor;
	int				lineWeight;

	COLORREF		transColor;
	WORD			pattern;

	bool			useTrans;

	int				polyn;
	char			polyCoords[1];
};

typedef struct tagEDATA_V2
{
	extHeader		eHeader;
	int				width;
	int				height;

	int				shape;
	int				fillType;
	int				gradDir;

	COLORREF		fillColor;
	COLORREF		fillColor2;
	COLORREF		lineColor;
	int				lineWeight;

	COLORREF		transColor;
	bool			useTrans;

	DWORD			imageOptions;

	//WORD			pattern;

	// V2
	COLORREF		textColor;
	LOGFONT			textFont;
	DWORD			textAlignFlags;
	DWORD			textEffect;
	DWORD			textCoef;

	size_t			polyn;

	size_t			patternCount;
	size_t			imageCount;
	size_t			textLen;
	size_t			polyCoordLen;

	size_t			patternOff;
	size_t			imageOff;
	size_t			textOff;
	size_t			polyCoordOff;

	size_t			endOff;

	//int				polyn;
	//char			polyCoords[1];
} EDITDATA;
typedef EDITDATA * LPEDATA;

void RecalcLayout (LPEDATA edPtr);
WORD* ptrPatterns (LPEDATA edPtr);
WORD* ptrImages (LPEDATA edPtr);
char* ptrText (LPEDATA edPtr);
char* ptrPolyCoords (LPEDATA edPtr);
