// ============================================================================
//
// This file contains routines that are handled only during the Edittime,
// under the Frame and Event editors.
//
// Including creating, display, and setting up your object.
// 
// ============================================================================

#include "common.h"
#include <sstream>

LPCSTR propTypeList[] = {
	0,
	"Rectangle",
	"Ellipse",
	"Polygon",
	NULL
};

LPCSTR propFillList[] = {
	0,
	"Solid Color",
	"Gradient",
	"Pattern",
	"Image",
	NULL
};


LPCSTR propGradDirList[] = {
	0,
	"Horizontal",
	"Vertical",
	NULL
};

LPCSTR propTextInk[] = {
	0,
	"None",
	"Semi-transparent",
	"Inverted",
	"XOR",
	"AND",
	"OR",
	"Monochrome",
	"Add",
	"Subtract",
	NULL
};

int minMaxLineWeight[] = { 0, 1000 };
int minMaxCoef[] = { 0, 128 };

void RecalcLayout_Text (LPEDATA &edPtr, mv _far *mV, size_t textLen);
void RecalcLayout_Poly (LPEDATA &edPtr, mv _far *mV, size_t polyLen);

// --------------------
// Properties
// --------------------

PROPS_IDS_START()

	PROPID_SIZE,
	PROPID_WIDTH,
	PROPID_HEIGHT,
	PROPID_DISPLAY,
	PROPID_SHAPE,
	PROPID_FILLCOLOR,
	PROPID_LINECOLOR,
	PROPID_LINEWEIGHT,
	PROPID_PATTERN,
	PROPID_IMAGE,

	PROPID_POLY,
	PROPID_POLYCOORDS,

	PROPID_TRANSCOLOR,
	PROPID_TRANSCUSTOM,

	PROPID_FILL,
	PROPID_FILLTYPE,
	PROPID_FILLCOLOR2,
	PROPID_GRADDIR,
	PROPID_IMGRESAMPLE,
	PROPID_IMGRESAMPLETRANS,

	PROPID_TEXT,
	PROPID_TEXTDATA,

	PROPID_TEXTINK,
	PROPID_TEXTEFFECT,
	PROPID_TEXTCOEF,
	PROPID_TEXTAA,

PROPS_IDS_END()

PROPS_DATA_START()

	PropData_Group(PROPID_DISPLAY, PSTR("Display"), PSTR("Display")),
	PropData_ComboBox(PROPID_SHAPE, PSTR("Shape"), PSTR("..."), propTypeList),
	PropData_Color(PROPID_LINECOLOR, PSTR("Line Color"), PSTR("...")),
	PropData_SpinEdit(PROPID_LINEWEIGHT, PSTR("Line Weight"), PSTR("..."), minMaxLineWeight),
	PropData_Group(PROPID_FILL, PSTR("Fill Options"), PSTR("Fill Options")),
	PropData_ComboBox(PROPID_FILLTYPE, PSTR("Fill"), PSTR("..."), propFillList),
	PropData_Group(PROPID_TEXT, PSTR("Text"), PSTR("Text")),
	PropData_EditString(PROPID_TEXTDATA, PSTR("String"), PSTR("...")),

PROPS_DATA_END()

PropData textEffect[] = {
	PropData_Group(PROPID_TEXTINK, PSTR("Ink Effect"), PSTR("Ink Effect")),
	PropData_ComboBox(PROPID_TEXTEFFECT, PSTR("Ink Effect"), PSTR("..."), propTextInk),
	PropData_SpinEdit(PROPID_TEXTCOEF, PSTR("Coefficient"), PSTR("..."), minMaxCoef),
	PropData_CheckBox(PROPID_TEXTAA, PSTR("Anti-aliasing"), PSTR("...")),
	PropData_End()
};

PropData polyData[] = {
	PropData_Group(PROPID_POLY, PSTR("Polygon"), PSTR("Polygon")),
	PropData_EditMultiLine(PROPID_POLYCOORDS, PSTR("Point List"), PSTR("Polygon coordinate list")),
	PropData_End()
};

PropData transColor[] = {
	PropData_Group(PROPID_TRANSCOLOR, PSTR("Transparent Color"), PSTR("Transparent Color")),
	PropData_Color_Check(PROPID_TRANSCUSTOM, PSTR("Custom"), PSTR("...")), 
	PropData_End()
};

PropData fillFlat[] = {
	PropData_Color(PROPID_FILLCOLOR, PSTR("Fill Color"), PSTR("...")),
	PropData_End()
};

PropData fillGradient[] = {
	PropData_Color(PROPID_FILLCOLOR, PSTR("Fill Color 1"), PSTR("...")),
	PropData_Color(PROPID_FILLCOLOR2, PSTR("Fill Color 2"), PSTR("...")),
	PropData_ComboBox(PROPID_GRADDIR, PSTR("Direction"), PSTR("..."), propGradDirList),
	PropData_End()
};

PropData fillPattern[] = {
	PropData_Color(PROPID_FILLCOLOR, PSTR("Fill Color"), PSTR("...")),
	PropData_Button(PROPID_PATTERN, PSTR("Fill Pattern"), PSTR("..."), PSTR("Edit")),
	PropData_End()
};

PropData fillImage[] = {
	PropData_Color(PROPID_FILLCOLOR, PSTR("Fill Color"), PSTR("...")),
	PropData_Button(PROPID_IMAGE, PSTR("Fill Image"), PSTR("..."), PSTR("Edit")),
	PropData_CheckBox(PROPID_IMGRESAMPLE, PSTR("Resample"), PSTR("...")),
	PropData_CheckBox(PROPID_IMGRESAMPLETRANS, PSTR("Resample Edge"), PSTR("...")),
	PropData_End()
};

// --------------------
// Debugger
// --------------------

DEBUGGER_IDS_START()

	// DB_CURRENTSTRING,

DEBUGGER_IDS_END()

DEBUGGER_ITEMS_START()

	//  DB_CURRENTSTRING,
	//	DB_CURRENTSTRING|DB_EDITABLE,
	//	DB_CURRENTVALUE|DB_EDITABLE,
	//	DB_CURRENTCHECK,
	//	DB_CURRENTCOMBO,

DEBUGGER_ITEMS_END()

// --------------------
// GetProperties
// --------------------
// Inserts properties into the properties of the object.
//

BOOL WINAPI DLLExport GetProperties(LPMV mV, LPEDATA edPtr, BOOL bMasterItem)
{
#ifndef RUN_ONLY
	mvInsertProps(mV, edPtr, Properties, PROPID_TAB_GENERAL, TRUE);
	mvInsertProps(mV, edPtr, transColor, PROPID_TAB_DISPLAY, TRUE);
	mvInsertProps(mV, edPtr, textEffect, PROPID_TAB_TEXTOPT, TRUE);

	if (edPtr->shape == TSHAPE_POLY) {
		mvInsertProps(mV, edPtr, polyData, PROPID_LINEWEIGHT, TRUE);
	}

	if (edPtr->fillType == TFILL_FLAT)
		mvInsertProps(mV, edPtr, fillFlat, PROPID_FILLTYPE, TRUE);
	else if (edPtr->fillType == TFILL_GRAD)
		mvInsertProps(mV, edPtr, fillGradient, PROPID_FILLTYPE, TRUE);
	else if (edPtr->fillType == TFILL_PATTERN)
		mvInsertProps(mV, edPtr, fillPattern, PROPID_FILLTYPE, TRUE);
	else if (edPtr->fillType == TFILL_IMAGE)
		mvInsertProps(mV, edPtr, fillImage, PROPID_FILLTYPE, TRUE);

#endif // !RUN_ONLY

	// OK
	return TRUE;
}

// --------------------
// ReleaseProperties
// --------------------
// Called when the properties are removed from the property window.
//

void WINAPI DLLExport ReleaseProperties(LPMV mV, LPEDATA edPtr, BOOL bMasterItem)
{
#ifndef RUN_ONLY
	// Write your code here
#endif // !RUN_ONLY
}

// --------------------
// GetPropCreateParam
// --------------------
// Called when a property is initialized and its creation parameter is NULL (in the PropData).
// Allows you, for example, to change the content of a combobox property according to specific settings in the EDITDATA structure.
//

LPARAM WINAPI DLLExport GetPropCreateParam(LPMV mV, LPEDATA edPtr, UINT nPropID)
{
#ifndef RUN_ONLY
	// Example
	// -------
//	if ( nPropID == PROPID_COMBO )
//	{
//		switch (edPtr->sType)
//		{
//		case TYPE1:
//			return (LPARAM)ComboList1;
//		case TYPE2:
//			return (LPARAM)ComboList2;
//		}
//	}
#endif // !RUN_ONLY
	return NULL;
}

// ----------------------
// ReleasePropCreateParam
// ----------------------
// Called after a property has been initialized.
// Allows you, for example, to free memory allocated in GetPropCreateParam.
//

void WINAPI DLLExport ReleasePropCreateParam(LPMV mV, LPEDATA edPtr, UINT nPropID, LPARAM lParam)
{
#ifndef RUN_ONLY
#endif // !RUN_ONLY
}

// --------------------
// GetPropValue
// --------------------
// Returns the value of properties that have a value.
// Note: see GetPropCheck for checkbox properties
//

LPVOID WINAPI DLLExport GetPropValue(LPMV mV, LPEDATA edPtr, UINT nPropID)
{
#ifndef RUN_ONLY

	switch (nPropID) {

	case PROPID_SHAPE:
		return new CPropDWordValue(edPtr->shape);

	case PROPID_FILLTYPE:
		return new CPropDWordValue(edPtr->fillType);

	case PROPID_GRADDIR:
		return new CPropDWordValue(edPtr->gradDir);

	case PROPID_TRANSCUSTOM:
		return new CPropDWordValue(edPtr->transColor);

	case PROPID_FILLCOLOR:
		return new CPropDWordValue(edPtr->fillColor);

	case PROPID_FILLCOLOR2:
		return new CPropDWordValue(edPtr->fillColor2);

	case PROPID_LINECOLOR:
		return new CPropDWordValue(edPtr->lineColor);

	case PROPID_LINEWEIGHT:
		return new CPropDWordValue(edPtr->lineWeight);

	case PROPID_POLYCOORDS:
		return new CPropDataValue(ptrPolyCoords(edPtr));

	case PROPID_TEXTDATA:
		return new CPropDataValue(ptrText(edPtr));

	case PROPID_TEXTEFFECT:
		return new CPropDWordValue(edPtr->textEffect & EFFECT_MASK);

	case PROPID_TEXTCOEF:
		return new CPropDWordValue(edPtr->textCoef);

	case PROPID_PATTERN:
		{
			CPropDataValue* pv = new CPropDataValue(2 * sizeof(WORD), NULL);
			if (pv != NULL) {
				if (pv->m_pData != NULL) {
					LPWORD pw = (LPWORD)pv->m_pData;
					*pw++ = 1;
					*pw++ = ptrPatterns(edPtr)[0];
					return pv;
				}
				pv->Delete();
			}
		}
		break;

	case PROPID_IMAGE:
		{
			CPropDataValue* pv = new CPropDataValue(2 * sizeof(WORD), NULL);
			if (pv != NULL) {
				if (pv->m_pData != NULL) {
					LPWORD pw = (LPWORD)pv->m_pData;
					*pw++ = 1;
					*pw++ = ptrImages(edPtr)[0];
					return pv;
				}
				pv->Delete();
			}
		}
		break;
	}

#endif // !RUN_ONLY
	return NULL;
}

// --------------------
// GetPropCheck
// --------------------
// Returns the checked state of properties that have a check box.
//

BOOL WINAPI DLLExport GetPropCheck(LPMV mV, LPEDATA edPtr, UINT nPropID)
{
#ifndef RUN_ONLY

	switch (nPropID) {
		case PROPID_TRANSCUSTOM:
			return edPtr->useTrans;

		case PROPID_IMGRESAMPLE:
			return edPtr->imageOptions & IMGOPT_RESAMPLE;

		case PROPID_IMGRESAMPLETRANS:
			return edPtr->imageOptions & IMGOPT_RESAMPLE_EDGE;

		case PROPID_TEXTAA:
			return edPtr->textEffect & EFFECTFLAG_ANTIALIAS;

		//case PROPID_PATTERN:
		//	return edPtr->usePattern;
	}
	
#endif // !RUN_ONLY
	return 0;		// Unchecked
}

// --------------------
// SetPropValue
// --------------------
// This routine is called by MMF after a property has been modified.
//

void WINAPI DLLExport SetPropValue(LPMV mV, LPEDATA edPtr, UINT nPropID, LPVOID lParam)
{
#ifndef RUN_ONLY
	// Gets the pointer to the CPropValue structure
	CPropValue* pValue = (CPropValue*)lParam;

	switch (nPropID) {

	case PROPID_TRANSCUSTOM:
		edPtr->transColor = ((CPropDWordValue*)pValue)->m_dwValue;
		break;

	case PROPID_FILLCOLOR:
		edPtr->fillColor = ((CPropDWordValue*)pValue)->m_dwValue;
		break;

	case PROPID_FILLCOLOR2:
		edPtr->fillColor2 = ((CPropDWordValue*)pValue)->m_dwValue;
		break;

	case PROPID_LINECOLOR:
		edPtr->lineColor = ((CPropDWordValue*)pValue)->m_dwValue;
		break;

	case PROPID_LINEWEIGHT:
		edPtr->lineWeight = ((CPropDWordValue*)pValue)->m_dwValue;
		if (edPtr->lineWeight < 0) {
			edPtr->lineWeight = 0;
		}
		break;

	case PROPID_GRADDIR:
		edPtr->gradDir = ((CPropDWordValue*)pValue)->m_dwValue;
		break;

	case PROPID_SHAPE:
		edPtr->shape = ((CPropDWordValue*)pValue)->m_dwValue;
		if (edPtr->shape == TSHAPE_POLY) {
			mvInsertProps(mV, edPtr, polyData, PROPID_LINEWEIGHT, TRUE);
		}
		else {
			mvRemoveProps(mV, edPtr, polyData);
		}
		break;

	case PROPID_FILLTYPE:
		edPtr->fillType = ((CPropDWordValue*)pValue)->m_dwValue;
		if (edPtr->fillType == TFILL_FLAT) {
			mvRemoveProps(mV, edPtr, fillGradient);
			mvRemoveProps(mV, edPtr, fillPattern);
			mvRemoveProps(mV, edPtr, fillImage);
			mvInsertProps(mV, edPtr, fillFlat, PROPID_FILLTYPE, TRUE);
		}
		else if (edPtr->fillType == TFILL_GRAD) {
			mvRemoveProps(mV, edPtr, fillFlat);
			mvRemoveProps(mV, edPtr, fillPattern);
			mvRemoveProps(mV, edPtr, fillImage);
			mvInsertProps(mV, edPtr, fillGradient, PROPID_FILLTYPE, TRUE);
		}
		else if (edPtr->fillType == TFILL_PATTERN) {
			mvRemoveProps(mV, edPtr, fillGradient);
			mvRemoveProps(mV, edPtr, fillFlat);
			mvRemoveProps(mV, edPtr, fillImage);
			mvInsertProps(mV, edPtr, fillPattern, PROPID_FILLTYPE, TRUE);
		}
		else if (edPtr->fillType == TFILL_IMAGE) {
			mvRemoveProps(mV, edPtr, fillGradient);
			mvRemoveProps(mV, edPtr, fillFlat);
			mvRemoveProps(mV, edPtr, fillPattern);
			mvInsertProps(mV, edPtr, fillImage, PROPID_FILLTYPE, TRUE);
		}
		break;

	case PROPID_TEXTDATA:
		{
			LPSTR pStr = (LPSTR)((CPropDataValue*)pValue)->m_pData;
			RecalcLayout_Text(edPtr, mV, strlen(pStr));
			strcpy(ptrText(edPtr), pStr);
		}		
		break;

	case PROPID_TEXTEFFECT:
		edPtr->textEffect = (edPtr->textEffect & ~EFFECT_MASK) | ((CPropDWordValue*)pValue)->m_dwValue & EFFECT_MASK;
		break;

	case PROPID_TEXTCOEF:
		edPtr->textCoef = ((CPropDWordValue*)pValue)->m_dwValue;
		break;

	case PROPID_POLYCOORDS:
		{
			LPSTR pStr = (LPSTR)((CPropDataValue*)pValue)->m_pData;
			RecalcLayout_Poly(edPtr, mV, strlen(pStr));
			strcpy(ptrPolyCoords(edPtr), pStr);

			// Get pair count
			edPtr->polyn = 0;
			std::stringstream str(pStr);
			while (str.good()) {
				double v;
				while (str.peek() == ',' || str.peek() == ' ') str.ignore(1);
				str >> v;
				edPtr->polyn++;
			}
			edPtr->polyn /= 2;
		}		
		break;

	case PROPID_PATTERN:
		if (((CPropDataValue*)pValue)->m_pData != NULL) {
			LPWORD pw = (LPWORD) ((CPropDataValue*)pValue)->m_pData;
			ptrPatterns(edPtr)[0] = *pw++;
		}
		break;

	case PROPID_IMAGE:
		if (((CPropDataValue*)pValue)->m_pData != NULL) {
			LPWORD pw = (LPWORD) ((CPropDataValue*)pValue)->m_pData;
			ptrImages(edPtr)[0] = *pw++;
		}
		break;

	//case PROPID_ITEM_ICON:
	//	break;
	}

	mvInvalidateObject(mV, edPtr);
	//mvRefreshProp(mV, edPtr, PROPID_ITEM_ICON, 1);

	// You may want to have your object redrawn in the frame editor after the modifications,
	// in this case, just call this function
	// mvInvalidateObject(mV, edPtr);

#endif // !RUN_ONLY
}

// --------------------
// SetPropCheck
// --------------------
// This routine is called by MMF when the user modifies a checkbox in the properties.
//

void WINAPI DLLExport SetPropCheck(LPMV mV, LPEDATA edPtr, UINT nPropID, BOOL nCheck)
{
#ifndef RUN_ONLY

	switch (nPropID) {

		case PROPID_TRANSCUSTOM:
			edPtr->useTrans = (nCheck != 0);
			break;

		case PROPID_IMGRESAMPLE:
			edPtr->imageOptions = (edPtr->imageOptions & ~IMGOPT_RESAMPLE) | (nCheck);
			break;

		case PROPID_IMGRESAMPLETRANS:
			edPtr->imageOptions = (edPtr->imageOptions & ~IMGOPT_RESAMPLE_EDGE) | (nCheck << 1);
			break;

		case PROPID_TEXTAA:
			edPtr->textEffect = (nCheck != 0) ? edPtr->textEffect | EFFECTFLAG_ANTIALIAS : edPtr->textEffect & ~EFFECTFLAG_ANTIALIAS;
			break;

		//case PROPID_PATTERN:
		//	edPtr->usePattern = (nCheck != 0);
		//	break;
	}

	mvInvalidateObject(mV, edPtr);
	// mvRefreshProp(mV, edPtr, PROPID_COMBO);

#endif // !RUN_ONLY
}

// --------------------
// EditProp
// --------------------
// This routine is called when the user clicks the button of a Button or EditButton property.
//

BOOL WINAPI DLLExport EditProp(LPMV mV, LPEDATA edPtr, UINT nPropID)
{
#ifndef RUN_ONLY

	switch (nPropID) {
		
	case PROPID_PATTERN:
		{
			EditAnimationParams eap;
			eap.m_dwSize = sizeof(EditAnimationParams);
			eap.m_pWindowTitle = NULL;
			eap.m_nImages = 1;
			eap.m_nMaxImages = 1;
			eap.m_nStartIndex = 0;
			eap.m_pImages = &ptrPatterns(edPtr)[0];
			eap.m_pImageTitles = NULL;
			eap.m_dwOptions = PICTEDOPT_FIXEDNFRAMES | PICTEDOPT_CANNOTMOVEFRAMES;
			//eap.m_dwFixedWidth = 16;
			//eap.m_dwFixedHeight = 16;
			
			if (mV->mvEditAnimation(edPtr, &eap, NULL)) {
				mvRefreshProp(mV, edPtr, PROPID_PATTERN, false);
				mvInvalidateObject(mV, edPtr);
				return true;
			}
		}
		break;

	case PROPID_IMAGE:
		{
			EditAnimationParams eap;
			eap.m_dwSize = sizeof(EditAnimationParams);
			eap.m_pWindowTitle = NULL;
			eap.m_nImages = 1;
			eap.m_nMaxImages = 1;
			eap.m_nStartIndex = 0;
			eap.m_pImages = &ptrImages(edPtr)[0];
			eap.m_pImageTitles = NULL;
			eap.m_dwOptions = PICTEDOPT_FIXEDNFRAMES | PICTEDOPT_CANNOTMOVEFRAMES;
			//eap.m_dwFixedWidth = 16;
			//eap.m_dwFixedHeight = 16;
			
			if (mV->mvEditAnimation(edPtr, &eap, NULL)) {
				mvRefreshProp(mV, edPtr, PROPID_IMAGE, false);
				mvInvalidateObject(mV, edPtr);
				return true;
			}
		}
		break;
	}

#endif // !RUN_ONLY
	return FALSE;
}

// --------------------
// IsPropEnabled
// --------------------
// This routine returns the enabled state of a property.
//

BOOL WINAPI IsPropEnabled(LPMV mV, LPEDATA edPtr, UINT nPropID)
{
#ifndef RUN_ONLY

	// Example
	// -------
/*
	switch (nPropID) {

	case PROPID_CHECK:
		return (edPtr->nComboIndex != 0);
	}
*/
#endif // !RUN_ONLY
	return TRUE;
}


// ============================================================================
//
// TEXT PROPERTIES
// 
// ============================================================================

// --------------------
// GetTextCaps
// --------------------
// Return the text capabilities of the object under the frame editor.
//

DWORD WINAPI DLLExport GetTextCaps(mv _far *mV, LPEDATA edPtr)
{
	return (TEXT_ALIGN_LEFT|TEXT_ALIGN_HCENTER|TEXT_ALIGN_RIGHT|TEXT_ALIGN_TOP|TEXT_ALIGN_VCENTER|TEXT_ALIGN_BOTTOM|TEXT_FONT|TEXT_COLOR);
}

// --------------------
// GetTextFont
// --------------------
// Return the font used the object.
// Note: the pStyle and cbSize parameters are obsolete and passed for compatibility reasons only.
//

BOOL WINAPI DLLExport GetTextFont(mv _far *mV, LPEDATA edPtr, LPLOGFONT plf, LPSTR pStyle, UINT cbSize)
{
#if !RUN_ONLY
	// Example: copy LOGFONT structure from EDITDATA
	memcpy(plf, &edPtr->textFont, sizeof(LOGFONT));
#endif // !RUN_ONLY

	return TRUE;
}

// --------------------
// SetTextFont
// --------------------
// Change the font used the object.
// Note: the pStyle parameter is obsolete and passed for compatibility reasons only.
//

BOOL WINAPI DLLExport SetTextFont(mv _far *mV, LPEDATA edPtr, LPLOGFONT plf, LPCSTR pStyle)
{
#if !RUN_ONLY
	// Example: copy LOGFONT structure to EDITDATA
	memcpy(&edPtr->textFont, plf, sizeof(LOGFONT));
#endif // !RUN_ONLY

	return TRUE;
}

// --------------------
// GetTextClr
// --------------------
// Get the text color of the object.
//

COLORREF WINAPI DLLExport GetTextClr(mv _far *mV, LPEDATA edPtr)
{
	// Example
	return edPtr->textColor;	// edPtr->fontColor;
}

// --------------------
// SetTextClr
// --------------------
// Set the text color of the object.
//

void WINAPI DLLExport SetTextClr(mv _far *mV, LPEDATA edPtr, COLORREF color)
{
	// Example
	edPtr->textColor = color;
}

// --------------------
// GetTextAlignment
// --------------------
// Get the text alignment of the object.
//

DWORD WINAPI DLLExport GetTextAlignment(mv _far *mV, LPEDATA edPtr)
{
	DWORD dw = 0;
#if !RUN_ONLY
	// Example
	// -------
	if ( (edPtr->textAlignFlags & DT_LEFT) != 0 )
		dw |= TEXT_ALIGN_LEFT;
	if ( (edPtr->textAlignFlags & DT_CENTER) != 0 )
		dw |= TEXT_ALIGN_HCENTER;
	if ( (edPtr->textAlignFlags & DT_RIGHT) != 0 )
		dw |= TEXT_ALIGN_RIGHT;
	if ( (edPtr->textAlignFlags & DT_TOP) != 0 )
		dw |= TEXT_ALIGN_TOP;
	if ( (edPtr->textAlignFlags & DT_VCENTER) != 0 )
		dw |= TEXT_ALIGN_VCENTER;
	if ( (edPtr->textAlignFlags & DT_BOTTOM) != 0 )
		dw |= TEXT_ALIGN_BOTTOM;

#endif // !RUN_ONLY
	return dw;
}

// --------------------
// SetTextAlignment
// --------------------
// Set the text alignment of the object.
//

void WINAPI DLLExport SetTextAlignment(mv _far *mV, LPEDATA edPtr, DWORD dwAlignFlags)
{
#if !RUN_ONLY
	// Example
	// -------
	DWORD dw = edPtr->textAlignFlags;

	if ( (dwAlignFlags & TEXT_ALIGN_LEFT) != 0 )
		dw = (dw & ~(DT_LEFT|DT_CENTER|DT_RIGHT)) | DT_LEFT;
	if ( (dwAlignFlags & TEXT_ALIGN_HCENTER) != 0 )
		dw = (dw & ~(DT_LEFT|DT_CENTER|DT_RIGHT)) | DT_CENTER;
	if ( (dwAlignFlags & TEXT_ALIGN_RIGHT) != 0 )
		dw = (dw & ~(DT_LEFT|DT_CENTER|DT_RIGHT)) | DT_RIGHT;

	if ( (dwAlignFlags & TEXT_ALIGN_TOP) != 0 )
		dw = (dw & ~(DT_TOP|DT_VCENTER|DT_BOTTOM)) | DT_TOP;
	if ( (dwAlignFlags & TEXT_ALIGN_VCENTER) != 0 )
		dw = (dw & ~(DT_TOP|DT_VCENTER|DT_BOTTOM)) | DT_VCENTER;
	if ( (dwAlignFlags & TEXT_ALIGN_BOTTOM) != 0 )
		dw = (dw & ~(DT_TOP|DT_VCENTER|DT_BOTTOM)) | DT_BOTTOM;

	edPtr->textAlignFlags = dw;

#endif // !RUN_ONLY
}


// -----------------
// BmpToImg
// -----------------
// Converts an image from the resource to an image displayable under MMF2
// Not used in this template, but it is a good example on how to create
// an image.
//


WORD BmpToImg(int bmID, npAppli idApp, short HotX = 0, short HotY = 0, short ActionX = 0, short ActionY = 0)
{
	Img					ifo;
	WORD				img;
	HRSRC				hs;
	HGLOBAL				hgBuf;
	LPBYTE				adBuf;
	LPBITMAPINFOHEADER	adBmi;

	img = 0;
	if ((hs = FindResource(hInstLib, MAKEINTRESOURCE(bmID), RT_BITMAP)) != NULL)
	{
		if ((hgBuf = LoadResource(hInstLib, hs)) != NULL)
		{
			if ((adBuf = (LPBYTE)LockResource(hgBuf)) != NULL)
			{
				adBmi = (LPBITMAPINFOHEADER)adBuf;
				ifo.imgXSpot = HotX;
				ifo.imgYSpot = HotY;
				ifo.imgXAction = ActionX;
				ifo.imgYAction = ActionY;
				if (adBmi->biBitCount > 4)
					RemapDib((LPBITMAPINFO)adBmi, idApp, NULL);
				img = (WORD)DibToImage(idApp, &ifo, adBmi);
				UnlockResource(hgBuf);
			}
			FreeResource(hgBuf);
		}
	}
	return img;
}


/*WORD SurfaceToImg (LPSURFACE psf, npAppli idApp, short HotX = 0, short HotY = 0, short ActionX = 0, short ActionY = 0)
{
	Img					ifo;
	WORD				img;
	LPBYTE				adBuf;
	LPBITMAPINFOHEADER	adBmi;

	img = 0;
	adBmi = (LPBITMAPINFOHEADER) ad*/

// ============================================================================
//
// ROUTINES USED UNDER FRAME EDITOR
// 
// ============================================================================

void DrawObject(cSurface& sf, mv _far *mV, LPEDATA edPtr)
{
	//LPSURFACE wSurf; // = WinGetSurface((int)mV->mvIdEditWin);
	//GetSurfacePrototype(&wSurf, 24, ST_MEMORYWITHDC, SD_DIB);

	//if (wSurf != NULL) {
		//int x = rc->left;
		//int y = rc->top;
		int w = edPtr->width; //rc->right - rc->left;
		int h = edPtr->height; //rc->bottom - rc->top;

		//if (!edPtr->pSf) {
		//	edPtr->pSf = new cSurface();
		//}

		//cSurface& sf = *(edPtr->pSf);
		//sf.Create(w, h, wSurf);
		//sf.Fill(RGB(0, 128, 0));

		// Transparent color
		if (edPtr->useTrans) {
			sf.Fill(edPtr->transColor);
			sf.SetTransparentColor(edPtr->transColor);
		}

		// Setup pattern if necessary
		CFillFlat* fdLine = 0;
		CFillData* fdFill = 0;
		cSurface* pattern = 0;
		switch (edPtr->fillType) {
			case TFILL_GRAD:
				fdLine = new CFillFlat(edPtr->lineColor);
				fdFill = (CFillData*) new CFillDir(edPtr->fillColor, edPtr->fillColor2, edPtr->gradDir == GRAD_VERT);
				break;

			case TFILL_PATTERN:
				{
					cSurface base;
					LockImageSurface(mV->mvIdAppli, ptrPatterns(edPtr)[0], base, LOCKIMAGE_ALLREADACCESS);

					pattern = new cSurface();
					pattern->Create(base.GetWidth(), base.GetHeight(), &sf);
					pattern->Fill(edPtr->fillColor);
					base.Blit(*pattern, 0, 0, BMODE_TRANSP);
					UnlockImageSurface(base);

					fdLine = new CFillFlat(edPtr->lineColor);
					fdFill = (CFillData*) new CFillMosaic(pattern);
				}
				break;
				
			case TFILL_IMAGE:
				{
					cSurface base;
					LockImageSurface(mV->mvIdAppli, ptrImages(edPtr)[0], base, LOCKIMAGE_ALLREADACCESS);

					cSurface imgbase;
					imgbase.Clone(base);

					DWORD opt = 0;
					BlitMode mode = BMODE_OPAQUE;
					if (edPtr->imageOptions & IMGOPT_RESAMPLE && edPtr->imageOptions & IMGOPT_RESAMPLE_EDGE)
						opt = STRF_RESAMPLE;
					else if (edPtr->imageOptions & IMGOPT_RESAMPLE) {
						opt = STRF_RESAMPLE_TRANSP;
						mode = BMODE_TRANSP;
					}

					pattern = new cSurface();
					pattern->Create(edPtr->width, edPtr->height, &sf);
					if (edPtr->imageOptions & IMGOPT_RESAMPLE_EDGE)
						imgbase.ReplaceColor(edPtr->fillColor, imgbase.GetTransparentColor());
					else
						pattern->Fill(edPtr->fillColor);
					imgbase.Stretch(*pattern, 0, 0, edPtr->width, edPtr->height, mode, BOP_COPY, 0, opt);
					UnlockImageSurface(base);

					fdLine = new CFillFlat(edPtr->lineColor);
					fdFill = (CFillData*) new CFillMosaic(pattern);
				}
				break;
		}

		// Rectangle Shape
		if (edPtr->shape == TSHAPE_RECT) {
			if (edPtr->fillType == TFILL_FLAT)
				sf.Rectangle(0, 0, w, h, edPtr->fillColor, edPtr->lineWeight, edPtr->lineColor, 1); 
			else
				sf.Rectangle(0, 0, w, h, fdFill, edPtr->lineWeight, fdLine, 0);
		}

		// Ellipse Shape
		else if (edPtr->shape == TSHAPE_ELLIPSE) {
			if (edPtr->fillType == TFILL_FLAT)
				sf.Ellipse(0, 0, w, h, edPtr->fillColor, edPtr->lineWeight, edPtr->lineColor, 1);
			else
				sf.Ellipse(0, 0, w, h, fdFill, edPtr->lineWeight, fdLine, 0);
		}

		// Polygon Shape
		else if (edPtr->shape == TSHAPE_POLY) {
			std::stringstream str(ptrPolyCoords(edPtr));
			tagPOINT* pl = new tagPOINT[edPtr->polyn];

			int preAdjust = (edPtr->lineWeight - 1) / 2;
			int postAdjust = edPtr->lineWeight;
			for (size_t i = 0; i < edPtr->polyn; i++) {
				double a, b;
				while (str.peek() == ',' || str.peek() == ' ') str.ignore(1);
				str >> a;
				while (str.peek() == ',' || str.peek() == ' ') str.ignore(1);
				str >> b;
				pl[i].x = preAdjust + a * (w - postAdjust + (w % 2));
				pl[i].y = preAdjust + b * (h - postAdjust + (h % 2));
			}

			if (edPtr->fillType == TFILL_FLAT)
				sf.Polygon(pl, edPtr->polyn, edPtr->fillColor, edPtr->lineWeight, edPtr->lineColor, 1);
			else
				sf.Polygon(pl, edPtr->polyn, fdFill, edPtr->lineWeight, fdLine, 0);
		}

		// Text
		if (edPtr->textLen > 0) {
			HFONT hFnt = CreateFontIndirect(&edPtr->textFont);

			BlitOp bop = BOP_COPY;
			switch (edPtr->textEffect & EFFECT_MASK) {
				case 1: bop = BOP_BLEND; break;
				case 2: bop = BOP_INVERT; break;
				case 3: bop = BOP_XOR; break;
				case 4: bop = BOP_AND; break;
				case 5: bop = BOP_OR; break;
				case 6: bop = BOP_MONO; break;
				case 7: bop = BOP_ADD; break;
				case 8: bop = BOP_SUB; break;
			}

			RECT rc;
			rc.left = 0; rc.top = 0;
			rc.right = edPtr->width; rc.bottom = edPtr->height;
			BOOL bAntiA = (edPtr->textEffect & EFFECTFLAG_ANTIALIAS) ? TRUE : FALSE;
			//sf.TextOutA(ptrText(edPtr), edPtr->textLen, 0, 0, edPtr->textAlignFlags, &rc, edPtr->textColor, hFnt);
			DWORD dtFlags = edPtr->textAlignFlags | DT_SINGLELINE | DT_NOPREFIX | DT_END_ELLIPSIS;
			sf.DrawTextA(ptrText(edPtr), edPtr->textLen, &rc, dtFlags, edPtr->textColor, hFnt, BMODE_TRANSP, bop, edPtr->textCoef, bAntiA);

			if (hFnt)
				DeleteObject(hFnt);
		}

		// Cleanup
		if (edPtr->fillType == TFILL_GRAD) {
			delete fdFill;
			delete fdLine;
		}
		else if (edPtr->fillType == TFILL_PATTERN || edPtr->fillType == TFILL_IMAGE) {
			delete fdFill;
			delete fdLine;
			delete pattern;
		}
	//}
}

// --------------------
// MakeIcon
// --------------------
// Called once object is created or modified, just after setup.
//
// Note: this function is optional. If it's not defined in your extension,
// MMF2 will load the EXO_ICON bitmap if it's defined in your resource file.
//
// If you need to draw the icon manually, remove the comments around this function and in the .def file.
//


int WINAPI DLLExport MakeIconEx ( mv _far *mV, cSurface* pIconSf, LPSTR lpName, fpObjInfo oiPtr, LPEDATA edPtr )
{
	int error = -1;
#ifndef RUN_ONLY
	//if ( pIconSf->LoadImage(hInstLib, EXO_ICON) != 0 )
	//	error = 0;
		//edPtr->pSf->Stretch(*pIconSf);
		LPSURFACE proto;
		GetSurfacePrototype(&proto, 24, ST_MEMORYWITHDC, SD_DIB);
		pIconSf->Create(edPtr->width, edPtr->height, proto);
		DrawObject(*pIconSf, mV, edPtr);
		error = 0;
#endif // !RUN_ONLY
	return error;
}

// Recalc Layouts

size_t EPSize (LPEDATA edPtr) {
	size_t newEPLen = sizeof(EDITDATA) 
		+ edPtr->patternCount * sizeof(WORD)
		+ edPtr->imageCount * sizeof(WORD)
		+ edPtr->polyCoordLen + 1
		+ edPtr->textLen + 1;
	return newEPLen;
}

void RecalcLayout (LPEDATA edPtr) {
	size_t offset = sizeof(EDITDATA);
	edPtr->patternOff = offset;
	offset += edPtr->patternCount * sizeof(WORD);
	edPtr->imageOff = offset;
	offset += edPtr->imageCount * sizeof(WORD);
	edPtr->textOff = offset;
	offset += edPtr->textLen + 1;
	edPtr->polyCoordOff = offset;
	offset += edPtr->polyCoordLen + 1;
	edPtr->endOff = offset;
}

void RecalcLayout_Pattern (LPEDATA &edPtr, mv _far *mV, size_t patternCount) {
	if (edPtr->patternCount == patternCount) return;

	int szDiff = (patternCount - edPtr->patternCount) * sizeof(WORD);
	LPEDATA pNewPtr = (LPEDATA)mvReAllocEditData(mV, edPtr, EPSize(edPtr) + szDiff);

	if (pNewPtr != NULL) {
		edPtr = pNewPtr;
		int szRem = edPtr->endOff - edPtr->imageOff;
		memcpy((char*)ptrImages(edPtr) + szDiff, (char*)ptrImages(edPtr), szRem);
	}

	edPtr->patternCount = patternCount;
	RecalcLayout(edPtr);
}

void RecalcLayout_Text (LPEDATA &edPtr, mv _far *mV, size_t textLen) {
	if (edPtr->textLen == textLen) return;

	int szDiff = (textLen - edPtr->textLen);
	LPEDATA pNewPtr = (LPEDATA)mvReAllocEditData(mV, edPtr, EPSize(edPtr) + szDiff);

	if (pNewPtr != NULL) {
		edPtr = pNewPtr;
		int szRem = edPtr->endOff - edPtr->polyCoordOff;
		memcpy((char*)ptrPolyCoords(edPtr) + szDiff, (char*)ptrPolyCoords(edPtr), szRem);
	}

	edPtr->textLen = textLen;
	RecalcLayout(edPtr);
}

void RecalcLayout_Poly (LPEDATA &edPtr, mv _far *mV, size_t polyLen) {
	if (edPtr->polyCoordLen == polyLen) return;

	int szDiff = (polyLen - edPtr->polyCoordLen);
	LPEDATA pNewPtr = (LPEDATA)mvReAllocEditData(mV, edPtr, EPSize(edPtr) + szDiff + 1);

	if (pNewPtr != NULL) {
		edPtr = pNewPtr;
		//int szRem = edPtr->end - (char*)edPtr->end;
		//memcpy((char*)edPtr->end + szDiff, (char*)edPtr->end, szRem);
	}

	edPtr->polyCoordLen = polyLen;
	RecalcLayout(edPtr);
}

WORD* ptrPatterns (LPEDATA edPtr) {
	WORD* v = (WORD*)((char*)edPtr + edPtr->patternOff);
	return v;
}

WORD* ptrImages (LPEDATA edPtr) {
	return (WORD*)((char*)edPtr + edPtr->imageOff);
}

char* ptrText (LPEDATA edPtr) {
	return (char*)((char*)edPtr + edPtr->textOff);
}

char* ptrPolyCoords (LPEDATA edPtr) {
	char* v = (char*)((char*)edPtr + edPtr->polyCoordOff);
	return v;
}

// --------------------
// CreateObject
// --------------------
// Called when you choose "Create new object". It should display the setup box 
// and initialize everything in the datazone.

int WINAPI DLLExport CreateObject(mv _far *mV, fpLevObj loPtr, LPEDATA edPtr)
{
#ifndef RUN_ONLY

	// Do some rSDK stuff
	#include "rCreateObject.h"

	/*LPEDATA pNewPtr = (LPEDATA)mvReAllocEditData(mV, edPtr, sizeof(EDITDATA)+strlen(pStr));
	if (pNewPtr != NULL) {
		edPtr = pNewPtr;
		strcpy(edPtr->polyCoords, pStr);
	}*/
	
	// Set default object settings
	edPtr->shape = TSHAPE_RECT;
	edPtr->fillType = TFILL_FLAT;
	edPtr->gradDir = GRAD_HORIZ;

	edPtr->fillColor = RGB(128, 128, 128);
	edPtr->fillColor2 = RGB(240, 240, 240);
	edPtr->lineColor = RGB(16, 16, 16);
	edPtr->lineWeight = 1;

	edPtr->useTrans = false;
	edPtr->transColor = RGB(0, 0, 0);
	edPtr->imageOptions = IMGOPT_RESAMPLE | IMGOPT_RESAMPLE_EDGE;

	edPtr->width = 128;
	edPtr->height = 128;

	edPtr->polyn = 4;

	char* polyStr = "0.5, 0\r\n1, 0.5\r\n0.5, 1\r\n0, 0.5";
	char* textStr = "";

	edPtr->textColor = RGB(16, 16, 16);
	edPtr->textAlignFlags = DT_CENTER | DT_VCENTER;
	mV->mvGetDefaultFont(&edPtr->textFont, NULL, 0);
	edPtr->textEffect = 0;
	edPtr->textCoef = 0;

	// Pointer data
	edPtr->patternCount = 1;
	edPtr->imageCount = 1;
	edPtr->polyCoordLen = strlen(polyStr);
	edPtr->textLen = strlen(textStr);

	size_t newEPLen = EPSize(edPtr);
	LPEDATA pNewPtr = (LPEDATA)mvReAllocEditData(mV, edPtr, newEPLen);
	if (pNewPtr != NULL) {
		edPtr = pNewPtr;
		RecalcLayout(edPtr);
	}

	ptrPatterns(edPtr)[0] = BmpToImg(EXO_DEF1, mV->mvIdAppli, 0, 0, 0, 0);
	ptrImages(edPtr)[0] = BmpToImg(EXO_DEF2, mV->mvIdAppli, 0, 0, 0, 0);
	strcpy(ptrText(edPtr), textStr);
	strcpy(ptrPolyCoords(edPtr), polyStr);

	return 0;	// No error

#endif // !RUN_ONLY

	// Error
	return -1;
}

// --------------------
// EditObject
// --------------------
// One of the option from the menu has been selected, and not a default menu option
// automatically handled by MMF2: this routine is then called.
//

BOOL WINAPI EditObject (mv _far *mV, fpObjInfo oiPtr, fpLevObj loPtr, LPEDATA edPtr)
{
#ifndef RUN_ONLY
	return TRUE;

#endif // !RUN_ONLY

	// Error
	return FALSE;
}

// --------------------
// SetEditSize
// --------------------
// Called by MMF2 when the object has been resized
//
// Note: remove the comments if your object can be resized (and remove the comments in the .def file)


BOOL WINAPI SetEditSize(LPMV mv, LPEDATA edPtr, int cx, int cy)
{
#ifndef RUN_ONLY
	edPtr->width = cx;
	edPtr->height = cy;
#endif // !RUN_ONLY
	return TRUE;	// OK
}


// --------------------
// PutObject
// --------------------
// Called when each individual object is dropped in the frame.
//

void WINAPI	DLLExport PutObject(mv _far *mV, fpLevObj loPtr, LPEDATA edPtr, ushort cpt)
{
#ifndef RUN_ONLY
#endif // !RUN_ONLY
}

// --------------------
// RemoveObject
// --------------------
// Called when each individual object is removed from the frame.
//

void WINAPI	DLLExport RemoveObject(mv _far *mV, fpLevObj loPtr, LPEDATA edPtr, ushort cpt)
{
#ifndef RUN_ONLY
	// Is the last object removed?
    if (0 == cpt)
	{
		// Do whatever necessary to remove our data
	}
#endif // !RUN_ONLY
}

// --------------------
// DuplicateObject
// --------------------
// Called when an object is created from another one (note: should be called CloneObject instead...)
//

void WINAPI DLLExport DuplicateObject(mv __far *mV, fpObjInfo oiPtr, LPEDATA edPtr)
{
#ifndef RUN_ONLY
#endif // !RUN_ONLY
}

// --------------------
// GetObjectRect
// --------------------
// Returns the size of the rectangle of the object in the frame editor.
//

void WINAPI DLLExport GetObjectRect(mv _far *mV, RECT FAR *rc, fpLevObj loPtr, LPEDATA edPtr)
{
#ifndef RUN_ONLY
	rc->right = rc->left + edPtr->width;	// edPtr->swidth;
	rc->bottom = rc->top + edPtr->height;	// edPtr->sheight;
#endif // !RUN_ONLY
	return;
}


// --------------------
// EditorDisplay
// --------------------
// Displays the object under the frame editor
//
// Note: this function is optional. If it's not defined in your extension,
// MMF2 will load and display the EXO_IMAGE bitmap if it's defined in your resource file.
//
// If you need to draw the icon manually, remove the comments around this function and in the .def file.
//

void WINAPI DLLExport EditorDisplay(mv _far *mV, fpObjInfo oiPtr, fpLevObj loPtr, LPEDATA edPtr, RECT FAR *rc)
{
#ifndef RUN_ONLY

	LPSURFACE wSurf = WinGetSurface((int)mV->mvIdEditWin);

	//LPSURFACE proto = NULL;
	//if (wSurf != NULL) {
	//	GetSurfacePrototype(&proto, wSurf->GetDepth(), ST_MEMORYWITHDC, SD_DIB);
	//}

	if (wSurf != NULL) {
		int x = rc->left;
		int y = rc->top;
		int w = rc->right - rc->left;
		int h = rc->bottom - rc->top;

		cSurface sf;
		sf.Create(w, h, wSurf);
		DrawObject(sf, mV, edPtr);

		// Set Ink Effect & Etc.
		BlitMode bm = (oiPtr->oiHdr.oiInkEffect & EFFECTFLAG_TRANSPARENT) ? BMODE_TRANSP : BMODE_OPAQUE;
		BOOL bAntiA = (oiPtr->oiHdr.oiInkEffect & EFFECTFLAG_ANTIALIAS) ? TRUE : FALSE;
		BlitOp bo = (BlitOp)(oiPtr->oiHdr.oiInkEffect & EFFECT_MASK);
		int effectParam = oiPtr->oiHdr.oiInkEffectParam;

		sf.Blit(*wSurf, x, y, bm, bo, effectParam, bAntiA ? BLTF_ANTIA : 0);
	}

	// This is a simple case of drawing an image onto MMF's frame editor window
	// First, we must get a pointer to the surface used by the frame editor

	/*LPSURFACE ps = WinGetSurface((int)mV->mvIdEditWin);
	if ( ps != NULL )		// Do the following if this surface exists
	{
		int x = rc->left;	// get our boundaries
		int y = rc->top;
		int w = rc->right-rc->left;
		int h = rc->bottom-rc->top;

		cSurface is;			// New surface variable for us to use
		is.Create(4, 4, ps);	// Create a surface implementation from a prototype (frame editor win)
		is.LoadImage(hInstLib, EXO_IMAGE, LI_REMAP);	// Load our bitmap from the resource,
														// and remap palette if necessary
		is.Blit(*ps, x, y, BMODE_TRANSP, BOP_COPY, 0);	// Blit the image to the frame editor surface!
		// This actually blits (or copies) the whole of our surface onto the frame editor's surface
		// at a specified position.
		// We could use different image effects when we copy, e.g. invert, AND, OR, XOR,
		// blend (semi-transparent, the 6th param is amount of transparency)
		// You can 'anti-alias' with the 7th param (default=0 or off)
	}*/

#endif // !RUN_ONLY
}


// --------------------
// IsTransparent
// --------------------
// This routine tells MMF2 if the mouse pointer is over a transparent zone of the object.
// 

extern "C" BOOL WINAPI DLLExport IsTransparent(mv _far *mV, fpLevObj loPtr, LPEDATA edPtr, int dx, int dy)
{
#ifndef RUN_ONLY
	// Write your code here
#endif // !RUN_ONLY
	return FALSE;
}

// --------------------
// PrepareToWriteObject
// --------------------
// Just before writing the datazone when saving the application, MMF2 calls this routine.
// 

void WINAPI	DLLExport PrepareToWriteObject(mv _far *mV, LPEDATA edPtr, fpObjInfo adoi)
{
#ifndef RUN_ONLY
	// Write your code here
#endif // !RUN_ONLY
}

// --------------------
// GetFilters
// --------------------
//

BOOL WINAPI GetFilters(LPMV mV, LPEDATA edPtr, DWORD dwFlags, LPVOID pReserved)
{
#ifndef RUN_ONLY
	// If your extension uses image filters
//	if ( (dwFlags & GETFILTERS_IMAGES) != 0 )
//		return TRUE;

	// If your extension uses sound filters
//	if ( (dwFlags & GETFILTERS_SOUNDS) != 0 )
//		return TRUE;
#endif // RUN_ONLY
	return FALSE;
}

// --------------------
// UsesFile
// --------------------
// Triggers when a file is dropped onto the frame
// Return TRUE if you can create an object from the given file
//

BOOL WINAPI	DLLExport UsesFile (LPMV mV, LPSTR fileName)
{
	BOOL r = FALSE;
#ifndef RUN_ONLY

	// Example: return TRUE if file extension is ".txt"
/*	
	LPSTR	ext, npath;

	if ( fileName != NULL )
	{
		if ( (ext=(LPSTR)malloc(_MAX_EXT)) != NULL )
		{
			if ( (npath=(LPSTR)malloc(_MAX_PATH)) != NULL )
			{
				strcpy(npath, fileName);
				_splitpath(npath, NULL, NULL, NULL, ext);
				if ( _stricmp(ext, ".txt") == 0 )
					r = TRUE;
				free(npath);
			}
			free(ext);
		}
	} */
#endif // !RUN_ONLY
	return r;
}


// --------------------
// CreateFromFile
// --------------------
// Creates a new object from file
//

void WINAPI	DLLExport CreateFromFile (LPMV mV, LPSTR fileName, LPEDATA edPtr)
{
#ifndef RUN_ONLY
	// Initialize your extension data from the given file
//	edPtr->swidth = 32;
//	edPtr->sheight = 32;

	// Example: store the filename
	// strcpy(edPtr->myFileName, fileName);
#endif // !RUN_ONLY
}



// ============================================================================
//
// ROUTINES USED UNDER EVENT / TIME / STEP-THROUGH EDITOR
// You should not need to change these routines
// 
// ============================================================================

// -----------------
// menucpy
// -----------------
// Internal routine used later, copy one menu onto another
// 
#ifndef RUN_ONLY
void menucpy(HMENU hTargetMenu, HMENU hSourceMenu)
{
	int			n, id, nMn;
	NPSTR		strBuf;
	HMENU		hSubMenu;

	nMn = GetMenuItemCount(hSourceMenu);
	strBuf = (NPSTR)LocalAlloc(LPTR, 80);
	for (n=0; n<nMn; n++)
	{
		if (0 == (id = GetMenuItemID(hSourceMenu, n)))
			AppendMenu(hTargetMenu, MF_SEPARATOR, 0, 0L);
		else
		{
			GetMenuString(hSourceMenu, n, strBuf, 80, MF_BYPOSITION);
			if (id != -1)
				AppendMenu(hTargetMenu, GetMenuState(hSourceMenu, n, MF_BYPOSITION), id, strBuf);
			else
			{
				hSubMenu = CreatePopupMenu();
				AppendMenu(hTargetMenu, MF_POPUP | MF_STRING, (uint)hSubMenu, strBuf);
				menucpy(hSubMenu, GetSubMenu(hSourceMenu, n));
			}
		}
	}
	LocalFree((HLOCAL)strBuf);
}

// -----------------
// GetPopupMenu
// -----------------
// Internal routine used later. Returns the first popup from a menu
// 
HMENU GetPopupMenu(LPEDATA edPtr,short mn)
{
	HMENU hPopup=CreatePopupMenu();
	
	if(mn == MN_CONDITIONS)
		menucpy(hPopup,ConditionMenu(edPtr));
	else if(mn == MN_ACTIONS)
		menucpy(hPopup,ActionMenu(edPtr));
	else if(mn == MN_EXPRESSIONS)
		menucpy(hPopup,ExpressionMenu(edPtr));

	return hPopup;
}

// --------------------
// GetEventInformations
// --------------------
// Internal routine used later. Look for one event in one of the eventInfos array...
// No protection to go faster: you must properly enter the conditions/actions!
//

static LPEVENTINFOS2 GetEventInformations(LPEVENTINFOS2 eiPtr, short code)

{
	while(eiPtr->infos.code != code)
		eiPtr = EVINFO2_NEXT(eiPtr);
	
	return eiPtr;
}
#endif // !RUN_ONLY


// ----------------------------------------------------
// GetConditionMenu / GetActionMenu / GetExpressionMenu
// ----------------------------------------------------
// Load the condition/action/expression menu from the resource, eventually
// enable or disable some options, and returns it to MMF2.
//

HMENU WINAPI DLLExport GetConditionMenu(mv _far *mV, fpObjInfo oiPtr, LPEDATA edPtr)
{
#ifndef RUN_ONLY
	// Check compatibility
	return GetPopupMenu(edPtr,MN_CONDITIONS);
#endif // !RUN_ONLY
	return NULL;
}

HMENU WINAPI DLLExport GetActionMenu(mv _far *mV, fpObjInfo oiPtr, LPEDATA edPtr)
{
#ifndef RUN_ONLY
	// Check compatibility
	return GetPopupMenu(edPtr,MN_ACTIONS);
#endif // !RUN_ONLY
	return NULL;
}

HMENU WINAPI DLLExport GetExpressionMenu(mv _far *mV, fpObjInfo oiPtr, LPEDATA edPtr)
{
#ifndef RUN_ONLY
	// Check compatibility
	return GetPopupMenu(edPtr,MN_EXPRESSIONS);
#endif // !RUN_ONLY
	return NULL;
}


// -------------------------------------------------------
// GetConditionTitle / GetActionTitle / GetExpressionTitle
// -------------------------------------------------------
// Returns the title of the dialog box displayed when entering
// parameters for the condition, action or expressions, if any.
// Here, we simply return the title of the menu option
//

#ifndef RUN_ONLY
void GetCodeTitle(LPEVENTINFOS2 eiPtr, short code, short param, short mn, LPSTR strBuf, WORD maxLen)
{
	HMENU		hMn;

	// Finds event in array
	eiPtr=GetEventInformations(eiPtr, code);

	// If a special string is to be returned
	short strID = EVINFO2_PARAMTITLE(eiPtr, param);

	if(strID) {
		switch(mn) {
		case MN_CONDITIONS:
			if(code>=0&&code<(short)Conditions.size()) {
				if(param>=0&&param<(short)Conditions[code]->getParamCount())
					strcpy(strBuf,Conditions[code]->getParamName(param));
			}
			break;
		case MN_ACTIONS:
			if(code>=0&&code<(short)Actions.size()) {
				if(param>=0&&param<(short)Actions[code]->getParamCount())
					strcpy(strBuf,Actions[code]->getParamName(param));
			}
			break;
		case MN_EXPRESSIONS:
			if(code>=0&&code<(short)Expressions.size()) {
				if(param>=0&&param<(short)Expressions[code]->getParamCount())
					strcpy(strBuf,Expressions[code]->getParamName(param));
			}
			break;
		}
	} else {
		if((hMn=LoadMenu(hInstLib, MAKEINTRESOURCE(mn)))) {
			GetMenuString(hMn, eiPtr->menu, strBuf, maxLen, MF_BYCOMMAND);
			DestroyMenu(hMn);
		}
	}
}
#else
#define GetCodeTitle(a,b,c,d,e,f)
#endif // !RUN_ONLY

void WINAPI DLLExport GetConditionTitle(mv _far *mV, short code, short param, LPSTR strBuf, short maxLen) {
	GetCodeTitle((LPEVENTINFOS2)conditionsInfos, code, param, MN_CONDITIONS, strBuf, maxLen);
}
void WINAPI DLLExport GetActionTitle(mv _far *mV, short code, short param, LPSTR strBuf, short maxLen) {
	GetCodeTitle((LPEVENTINFOS2)actionsInfos, code, param, MN_ACTIONS, strBuf, maxLen);
}
void WINAPI DLLExport GetExpressionTitle(mv _far *mV, short code, LPSTR strBuf, short maxLen) {
	GetCodeTitle((LPEVENTINFOS2)expressionsInfos, code, 0, MN_EXPRESSIONS, strBuf, maxLen);
}

// -------------------------------------------------------
// GetConditionTitle / GetActionTitle / GetExpressionTitle
// -------------------------------------------------------
// From a menu ID, these routines returns the code of the condition,
// action or expression, as defined in the .H file
//

short WINAPI DLLExport GetConditionCodeFromMenu(mv _far *mV, short menuId)
{
#ifndef RUN_ONLY
	LPEVENTINFOS2 eiPtr;
	int n;
	for(n=Conditions.size(),eiPtr=(LPEVENTINFOS2)conditionsInfos;n>0&&eiPtr->menu!=menuId;n--)
		eiPtr=EVINFO2_NEXT(eiPtr);
	if(n>0)
		return eiPtr->infos.code;
#endif // !RUN_ONLY
	return -1;
}

short WINAPI DLLExport GetActionCodeFromMenu(mv _far *mV, short menuId)
{
#ifndef RUN_ONLY
	LPEVENTINFOS2 eiPtr;
	int n;

	for (n=Actions.size(),eiPtr=(LPEVENTINFOS2)actionsInfos;n>0&&eiPtr->menu!=menuId;n--)
		eiPtr = EVINFO2_NEXT(eiPtr);
	if (n>0) 
		return eiPtr->infos.code;
#endif // !RUN_ONLY
	return -1;
}

short WINAPI DLLExport GetExpressionCodeFromMenu(mv _far *mV, short menuId)
{
#ifndef RUN_ONLY
	LPEVENTINFOS2 eiPtr;
	int	n;
	for (n=Expressions.size(),eiPtr=(LPEVENTINFOS2)expressionsInfos;n>0&&eiPtr->menu!=menuId;n--)
		eiPtr = EVINFO2_NEXT(eiPtr);
	if (n>0) 
		return eiPtr->infos.code;
#endif // !RUN_ONLY
	return -1;
}


// -------------------------------------------------------
// GetConditionInfos / GetActionInfos / GetExpressionInfos
// -------------------------------------------------------
// From a action / condition / expression code, returns 
// an infosEvents structure. 
//

LPINFOEVENTSV2 WINAPI DLLExport GetConditionInfos(mv _far *mV, short code)
{
#ifndef RUN_ONLY
	return &GetEventInformations((LPEVENTINFOS2)conditionsInfos, code)->infos;
#else
	return NULL;
#endif // !RUN_ONLY
}

LPINFOEVENTSV2 WINAPI DLLExport GetActionInfos(mv _far *mV, short code)
{
#ifndef RUN_ONLY
	return &GetEventInformations((LPEVENTINFOS2)actionsInfos, code)->infos;
#else
	return NULL;
#endif // !RUN_ONLY
}

LPINFOEVENTSV2 WINAPI DLLExport GetExpressionInfos(mv _far *mV, short code)
{
#ifndef RUN_ONLY
	return &GetEventInformations((LPEVENTINFOS2)expressionsInfos, code)->infos;
#else
	return NULL;
#endif // !RUN_ONLY
}


// ----------------------------------------------------------
// GetConditionString / GetActionString / GetExpressionString
// ----------------------------------------------------------
// From a action / condition / expression code, returns 
// the string to use for displaying it under the event editor
//

void WINAPI DLLExport GetConditionString(mv _far *mV, short code, LPSTR strPtr, short maxLen)
{
#ifndef RUN_ONLY
		if(code>=0&&code<(short)Conditions.size())
			strcpy(strPtr,Conditions[code]->getName());
#endif // !RUN_ONLY
}

void WINAPI DLLExport GetActionString(mv _far *mV, short code, LPSTR strPtr, short maxLen)
{
#ifndef RUN_ONLY
		if(code>=0&&code<(short)Actions.size())
			strcpy(strPtr,Actions[code]->getName());
#endif // !RUN_ONLY
}

void WINAPI DLLExport GetExpressionString(mv _far *mV, short code, LPSTR strPtr, short maxLen)
{
#ifndef RUN_ONLY
		if(code>=0&&code<(short)Expressions.size())
			strcpy(strPtr,Expressions[code]->getName());
#endif // !RUN_ONLY
}

// ----------------------------------------------------------
// GetExpressionParam
// ----------------------------------------------------------
// Returns the parameter name to display in the expression editor
//

void WINAPI DLLExport GetExpressionParam(mv _far *mV, short code, short param, LPSTR strBuf, short maxLen)
{
#ifndef RUN_ONLY
	if(strlen(Expressions[code]->getParamName(param)))
		strcpy(strBuf,Expressions[code]->getParamName(param));
	else
		*strBuf=0;
#endif
}

// ----------------------------------------------------------
// Custom Parameters
// ----------------------------------------------------------

// --------------------
// InitParameter
// --------------------
// Initialize the parameter.
//

void WINAPI InitParameter(mv _far *mV, short code, paramExt* pExt)
{
#if !RUN_ONLY
	// Example
	// -------
	// strcpy(&pExt->pextData[0], "Parameter Test");
	// pExt->pextSize = sizeof(paramExt) + strlen(pExt->pextData)+1;
#endif // !RUN_ONLY
}

// Example of custom parameter setup proc
// --------------------------------------
/*
#if !RUN_ONLY
BOOL CALLBACK DLLExport SetupProc(HWND hDlg, UINT msgType, WPARAM wParam, LPARAM lParam)
{
	paramExt*			pExt;

	switch (msgType)
	{
		case WM_INITDIALOG: // Init dialog

			// Save edptr
			SetWindowLong(hDlg, DWL_USER, lParam);
			pExt=(paramExt*)lParam;

			SetDlgItemText(hDlg, IDC_EDIT, pExt->pextData);
			return TRUE;

		case WM_COMMAND: // Command

			// Retrieve edptr
			pExt = (paramExt *)GetWindowLong(hDlg, DWL_USER);

			switch (wmCommandID)
			{
			case IDOK:	// Exit
				GetDlgItemText(hDlg, IDC_EDIT, pExt->pextData, 500);
				pExt->pextSize=sizeof(paramExt)+strlen(pExt->pextData)+1;
				EndDialog(hDlg, TRUE);
				return TRUE;

				default:
					break;
			}
			break;

		default:
			break;
	}
	return FALSE;
}
#endif // !RUN_ONLY
*/

// --------------------
// EditParameter
// --------------------
// Edit the parameter.
//

void WINAPI EditParameter(mv _far *mV, short code, paramExt* pExt)
{
#if !RUN_ONLY

	// Example
	// -------
	// DialogBoxParam(hInstLib, MAKEINTRESOURCE(DB_TRYPARAM), mV->mvHEditWin, SetupProc, (LPARAM)(LPBYTE)pExt);

#endif // !RUN_ONLY
}

// --------------------
// GetParameterString
// --------------------
// Initialize the parameter.
//

void WINAPI GetParameterString(mv _far *mV, short code, paramExt* pExt, LPSTR pDest, short size)
{
#if !RUN_ONLY

	// Example
	// -------
	// wsprintf(pDest, "Super parameter %s", pExt->pextData);

#endif // !RUN_ONLY
}

// -----------------
// GetObjInfos
// -----------------
// Return object info
//
// Info displayed in the object's About properties
// Note: ObjComment is also displayed in the Quick Description box in the Insert Object dialog box
//

void WINAPI	DLLExport GetObjInfos (mv _far *mV, LPEDATA edPtr, LPSTR ObjName, LPSTR ObjAuthor, LPSTR ObjCopyright, LPSTR ObjComment, LPSTR ObjHttp)
{
#ifndef RUN_ONLY
	strcpy(ObjName,ObjectName);
	strcpy(ObjAuthor,ObjectAuthor);
	strcpy(ObjCopyright,ObjectCopyright);
	strcpy(ObjComment,ObjectComment);
	strcpy(ObjHttp,ObjectURL);
#endif
}

// -----------------
// GetHelpFileName
// -----------------
// Returns the help filename of the object.
//

LPCSTR WINAPI GetHelpFileName()
{
#ifndef RUN_ONLY
	return ObjectHelp;
#else
	return NULL;
#endif
}

// -----------------
// GetRegID
// -----------------
// Returns the registered ID of the object
//

int WINAPI GetRegID()
{
#ifndef RUN_ONLY
	return ObjectRegID;
#else
	return REGID_RUNTIME;
#endif
}
