// ============================================================================
//
// This file contains the actions, conditions and expressions your object uses
// 
// ============================================================================

#include "common.h"

// ============================================================================
//
// CONDITIONS
// 
// ============================================================================

CONDITION(
	/* ID */			0,
	/* Name */			"%o is a rectangle",
	/* Flags */			EVFLAGS_ALWAYS,
	/* Params */		(0)
) {
	return rdPtr->shape == TSHAPE_RECT;
}

CONDITION(
	/* ID */			1,
	/* Name */			"%o is an ellipse",
	/* Flags */			EVFLAGS_ALWAYS,
	/* Params */		(0)
) {
	return rdPtr->shape == TSHAPE_ELLIPSE;
}

CONDITION(
	/* ID */			2,
	/* Name */			"%o is a polygon",
	/* Flags */			EVFLAGS_ALWAYS,
	/* Params */		(0)
) {
	return rdPtr->shape == TSHAPE_POLY;
}

// ============================================================================
//
// ACTIONS
// 
// ============================================================================

ACTION(
	/* ID */			0,
	/* Name */			"Set Size to (%0, %1)",
	/* Flags */			0,
	/* Params */		(2,PARAM_NUMBER,"Width",PARAM_NUMBER,"Height")
) {
	int p1 = (int) Param(TYPE_INT);
	int p2 = (int) Param(TYPE_INT);

	rdPtr->width = (p1 < 1) ? 1 : p1;
	rdPtr->height = (p2 < 1) ? 1 : p2;

	//rdPtr->rHo.hoImgWidth = p1;
	//rdPtr->rHo.hoImgHeight = p2;

	rdPtr->rebuildPoly = 1;
	rdPtr->rebuild = 1;
}

ACTION(
	/* ID */			1,
	/* Name */			"Set primary fill color to %0",
	/* Flags */			0,
	/* Params */		(1,PARAM_COLOUR,"System Color Index")
) {
	int p1 = (int) Param(TYPE_INT);

	rdPtr->fillColor = p1;
	rdPtr->rebuild = 1;
}

ACTION(
	/* ID */			2,
	/* Name */			"Set line color to %0",
	/* Flags */			0,
	/* Params */		(1,PARAM_COLOUR,"System Color Index")
) {
	int p1 = (int) Param(TYPE_INT);

	rdPtr->lineColor = p1;
	rdPtr->rebuild = 1;
}

ACTION(
	/* ID */			3,
	/* Name */			"Set line weight to %0",
	/* Flags */			0,
	/* Params */		(1,PARAM_NUMBER,"Line Weight")
) {
	int p1 = (int) Param(TYPE_INT);

	rdPtr->lineWeight = (p1 < 0) ? 0 : p1;
	rdPtr->rebuildPoly = 1;
	rdPtr->rebuild = 1;
}

ACTION(
	/* ID */			4,
	/* Name */			"Set secondary fill color to %0",
	/* Flags */			0,
	/* Params */		(1,PARAM_COLOUR,"System Color Index")
) {
	int p1 = (int) Param(TYPE_INT);

	rdPtr->fillColor2 = p1;
	rdPtr->rebuild = 1;
}

ACTION(
	/* ID */			5,
	/* Name */			"Set gradient (Direction = %0)",
	/* Flags */			0,
	/* Params */		(1,PARAM_NUMBER,"0 = Horizontal, 1 = Vertical")
) {
	int p1 = (int) Param(TYPE_INT);

	rdPtr->gradDir = (p1 == 1);
	rdPtr->rebuild = 1;
}

ACTION(
	/* ID */			6,
	/* Name */			"Set shape to Rectangle",
	/* Flags */			0,
	/* Params */		(0)
) {
	rdPtr->shape = TSHAPE_RECT;
	rdPtr->rebuild = 1;
}

ACTION(
	/* ID */			7,
	/* Name */			"Set shape to Ellipse",
	/* Flags */			0,
	/* Params */		(0)
) {
	rdPtr->shape = TSHAPE_ELLIPSE;
	rdPtr->rebuild = 1;
}

ACTION(
	/* ID */			8,
	/* Name */			"Set shape to Polygon",
	/* Flags */			0,
	/* Params */		(0)
) {
	rdPtr->shape = TSHAPE_POLY;
	rdPtr->rebuild = 1;
}

ACTION(
	/* ID */			9,
	/* Name */			"Set fill to Solid Color",
	/* Flags */			0,
	/* Params */		(0)
) {
	rdPtr->fillType = TFILL_FLAT;
	rdPtr->rebuild = 1;
}

ACTION(
	/* ID */			10,
	/* Name */			"Set fill to Gradient",
	/* Flags */			0,
	/* Params */		(0)
) {
	rdPtr->fillType = TFILL_GRAD;
	rdPtr->rebuild = 1;
}

ACTION(
	/* ID */			11,
	/* Name */			"Set fill to Pattern",
	/* Flags */			0,
	/* Params */		(0)
) {
	rdPtr->fillType = TFILL_PATTERN;
	rdPtr->rebuild = 1;
}

ACTION(
	/* ID */			12,
	/* Name */			"Load point set (%0)",
	/* Flags */			0,
	/* Params */		(1,PARAM_STRING,"Point set")
) {
	char* p1 = (char*) Param(TYPE_STRING);
	
	if (rdPtr->polyCoords)
		delete[] rdPtr->polyCoords;

	rdPtr->polyCoords = new char[strlen(p1) + 1];
	strncpy(rdPtr->polyCoords, p1, strlen(p1));

	rdPtr->rebuildPoly = 1;
}

ACTION(
	/* ID */			13,
	/* Name */			"Set angle to %0 (Quality = %1)",
	/* Flags */			0,
	/* Params */		(2,PARAM_NUMBER,"Enter angle in degrees between 0 and 359",PARAM_NUMBER,"Enter 0 (maximum speed), or 1 (maximum quality)")
) {
	int p1 = (int) Param(TYPE_INT);
	int p2 = ((int) Param(TYPE_INT) == 1) ? 1 : 0;

	rdPtr->rc.rcOldAngle = rdPtr->rc.rcAngle;
	rdPtr->rc.rcAngle = p1 % 360;
	//rdPtr->qRotation = (p2 == 1);
	rdPtr->rs.rsFlags = (rdPtr->rs.rsFlags & ~RSFLAG_ROTATE_ANTIA) | (p2 << 4);
	rdPtr->rebuildTrans = 1;
}

ACTION(
	/* ID */			14,
	/* Name */			"Set Hot Spot to (%0, %1)",
	/* Flags */			0,
	/* Params */		(2,PARAM_NUMBER,"X coordinate",PARAM_NUMBER,"Y coordinate")
) {
	int p1 = (int) Param(TYPE_INT);
	int p2 = (int) Param(TYPE_INT);

	rdPtr->hotX = p1;
	rdPtr->hotY = p2;

	//rdPtr->rebuild = 1;
	rdPtr->rebuildTrans = 1;
}

ACTION(
	/* ID */			15,
	/* Name */			"Set Hot Spot to top-left",
	/* Flags */			0,
	/* Params */		(0)
) {
	rdPtr->hotX = 0;
	rdPtr->hotY = 0;
	//rdPtr->rebuild = 1;
	rdPtr->rebuildTrans = 1;
}

ACTION(
	/* ID */			16,
	/* Name */			"Set Hot Spot to top-center",
	/* Flags */			0,
	/* Params */		(0)
) {
	rdPtr->hotX = rdPtr->width / 2;
	rdPtr->hotY = 0;
	//rdPtr->rebuild = 1;
	rdPtr->rebuildTrans = 1;
}

ACTION(
	/* ID */			17,
	/* Name */			"Set Hot Spot to top-right",
	/* Flags */			0,
	/* Params */		(0)
) {
	rdPtr->hotX = rdPtr->width;
	rdPtr->hotY = 0;
	//rdPtr->rebuild = 1;
	rdPtr->rebuildTrans = 1;
}

ACTION(
	/* ID */			18,
	/* Name */			"Set Hot Spot to left-center",
	/* Flags */			0,
	/* Params */		(0)
) {
	rdPtr->hotX = 0;
	rdPtr->hotY = rdPtr->height / 2;
	//rdPtr->rebuild = 1;
	rdPtr->rebuildTrans = 1;
}

ACTION(
	/* ID */			19,
	/* Name */			"Set Hot Spot to center",
	/* Flags */			0,
	/* Params */		(0)
) {
	int oldHotX = rdPtr->hotX;
	int oldHotY = rdPtr->hotY;
	rdPtr->hotX = rdPtr->width / 2;
	rdPtr->hotY = rdPtr->height / 2;
	int mvX = rdPtr->rHo.hoX + (rdPtr->hotX - oldHotX);
	int mvY = rdPtr->rHo.hoY + (rdPtr->hotY - oldHotY);
	rdPtr->rHo.hoAdRunHeader->rh4.rh4KpxFunctions[RFUNCTION_SETPOSITION].routine(&(rdPtr->rHo), mvX, mvY);
	//rdPtr->rebuild = 1;
	rdPtr->rebuildTrans = 1;
}

ACTION(
	/* ID */			20,
	/* Name */			"Set Hot Spot to right-center",
	/* Flags */			0,
	/* Params */		(0)
) {
	rdPtr->hotX = rdPtr->width;
	rdPtr->hotY = rdPtr->height / 2;
	//rdPtr->rebuild = 1;
	rdPtr->rebuildTrans = 1;
}

ACTION(
	/* ID */			21,
	/* Name */			"Set Hot Spot to bottom-left",
	/* Flags */			0,
	/* Params */		(0)
) {
	rdPtr->hotX = 0;
	rdPtr->hotY = rdPtr->height;
	//rdPtr->rebuild = 1;
	rdPtr->rebuildTrans = 1;
}

ACTION(
	/* ID */			22,
	/* Name */			"Set Hot Spot to bottom-center",
	/* Flags */			0,
	/* Params */		(0)
) {
	rdPtr->hotX = rdPtr->width / 2;
	rdPtr->hotY = rdPtr->height;
	//rdPtr->rebuild = 1;
	rdPtr->rebuildTrans = 1;
}

ACTION(
	/* ID */			23,
	/* Name */			"Set Hot Spot to bottom-right",
	/* Flags */			0,
	/* Params */		(0)
) {
	rdPtr->hotX = rdPtr->width;
	rdPtr->hotY = rdPtr->height;
	//rdPtr->rebuild = 1;
	rdPtr->rebuildTrans = 1;
}

ACTION(
	/* ID */			24,
	/* Name */			"Set scale to %0 (Quality = %1)",
	/* Flags */			0,
	/* Params */		(2,PARAM_NUMBER,"Enter scale value - 1.0 = normal dimension",PARAM_NUMBER,"Enter 0 (maximum speed), or 1 (maximum quality)")
) {
	long p1temp = CNC_GetFloatParameter(rdPtr);
	float p1 = *(float*) &p1temp;
	int p2 = ((int) Param(TYPE_INT) == 1) ? 1 : 0;

	rdPtr->rc.rcScaleX = p1;
	rdPtr->rc.rcScaleY = p1;
	//rdPtr->qScale = (p2 == 1);
	rdPtr->rs.rsFlags = (rdPtr->rs.rsFlags & ~RSFLAG_SCALE_RESAMPLE) | (p2 << 3);
	rdPtr->rebuildTrans = 1;
}

ACTION(
	/* ID */			25,
	/* Name */			"Set X scale to %0 (Quality = %1)",
	/* Flags */			0,
	/* Params */		(2,PARAM_NUMBER,"Enter scale value - 1.0 = normal dimension",PARAM_NUMBER,"Enter 0 (maximum speed), or 1 (maximum quality)")
) {
	long p1temp = CNC_GetFloatParameter(rdPtr);
	float p1 = *(float*) &p1temp;
	int p2 = ((int) Param(TYPE_INT) == 1) ? 1 : 0;

	rdPtr->rc.rcScaleX = p1;
	//rdPtr->qScale = (p2 == 1);
	rdPtr->rs.rsFlags = (rdPtr->rs.rsFlags & ~RSFLAG_SCALE_RESAMPLE) | (p2 << 3);
	rdPtr->rebuildTrans = 1;
}

ACTION(
	/* ID */			26,
	/* Name */			"Set Y scale to %0 (Quality = %1)",
	/* Flags */			0,
	/* Params */		(2,PARAM_NUMBER,"Enter scale value - 1.0 = normal dimension",PARAM_NUMBER,"Enter 0 (maximum speed), or 1 (maximum quality)")
) {
	long p1temp = CNC_GetFloatParameter(rdPtr);
	float p1 = *(float*) &p1temp;
	int p2 = ((int) Param(TYPE_INT) == 1) ? 1 : 0;

	rdPtr->rc.rcScaleY = p1;
	//rdPtr->qScale = (p2 == 1);
	rdPtr->rs.rsFlags = (rdPtr->rs.rsFlags & ~RSFLAG_SCALE_RESAMPLE) | (p2 << 3);
	rdPtr->rebuildTrans = 1;
}

ACTION(
	/* ID */			27,
	/* Name */			"Set fill to Image (Quality = %0)",
	/* Flags */			0,
	/* Params */		(1,PARAM_NUMBER,"Enter 0 (no resampling), 1 (resample image), or 2 (resample image + edge)")
) {
	int p1 = (int) Param(TYPE_INT);

	rdPtr->fillType = TFILL_IMAGE;
	switch (p1) {
		case 0: rdPtr->imageOptions = 0; break;
		case 1: rdPtr->imageOptions = IMGOPT_RESAMPLE; break;
		case 2: rdPtr->imageOptions = IMGOPT_RESAMPLE | IMGOPT_RESAMPLE_EDGE; break;
	}
	rdPtr->rebuild = 1;
}

ACTION(
	/* ID */			28,
	/* Name */			"Set Text (%0)",
	/* Flags */			0,
	/* Params */		(1,PARAM_STRING,"Text")
) {
	char* p1 = (char*) Param(TYPE_STRING);
	
	if (rdPtr->text)
		delete[] rdPtr->text;

	rdPtr->text = new char[strlen(p1) + 1];
	strncpy(rdPtr->text, p1, strlen(p1)+1);

	rdPtr->rebuild = 1;
}

ACTION(
	/* ID */			29,
	/* Name */			"Set horizontal text alignment to Left",
	/* Flags */			0,
	/* Params */		(0)
) {
	rdPtr->textAlignFlags = (rdPtr->textAlignFlags & ~(DT_LEFT | DT_CENTER | DT_RIGHT)) | DT_LEFT;
	rdPtr->rebuild = 1;
}

ACTION(
	/* ID */			30,
	/* Name */			"Set horizontal text alignment to Center",
	/* Flags */			0,
	/* Params */		(0)
) {
	rdPtr->textAlignFlags = (rdPtr->textAlignFlags & ~(DT_LEFT | DT_CENTER | DT_RIGHT)) | DT_CENTER;
	rdPtr->rebuild = 1;
}

ACTION(
	/* ID */			31,
	/* Name */			"Set horizontal text alignment to Right",
	/* Flags */			0,
	/* Params */		(0)
) {
	rdPtr->textAlignFlags = (rdPtr->textAlignFlags & ~(DT_LEFT | DT_CENTER | DT_RIGHT)) | DT_RIGHT;
	rdPtr->rebuild = 1;
}

ACTION(
	/* ID */			32,
	/* Name */			"Set vertical text alignment to Top",
	/* Flags */			0,
	/* Params */		(0)
) {
	rdPtr->textAlignFlags = (rdPtr->textAlignFlags & ~(DT_TOP| DT_VCENTER | DT_BOTTOM)) | DT_TOP;
	rdPtr->rebuild = 1;
}

ACTION(
	/* ID */			33,
	/* Name */			"Set vertical text alignment to Center",
	/* Flags */			0,
	/* Params */		(0)
) {
	rdPtr->textAlignFlags = (rdPtr->textAlignFlags & ~(DT_TOP| DT_VCENTER | DT_BOTTOM)) | DT_VCENTER;
	rdPtr->rebuild = 1;
}

ACTION(
	/* ID */			34,
	/* Name */			"Set vertical text alignment to Bottom",
	/* Flags */			0,
	/* Params */		(0)
) {
	rdPtr->textAlignFlags = (rdPtr->textAlignFlags & ~(DT_TOP| DT_VCENTER | DT_BOTTOM)) | DT_BOTTOM;
	rdPtr->rebuild = 1;
}

// ============================================================================
//
// EXPRESSIONS
// 
// ============================================================================


EXPRESSION(
	/* ID */			0,
	/* Name */			"Width(",
	/* Flags */			0,
	/* Params */		(0)
) {
	ReturnInteger(rdPtr->width);
}

EXPRESSION(
	/* ID */			1,
	/* Name */			"Height(",
	/* Flags */			0,
	/* Params */		(0)
) {
	ReturnInteger(rdPtr->height);
}

EXPRESSION(
	/* ID */			2,
	/* Name */			"FillColor(",
	/* Flags */			0,
	/* Params */		(0)
) {
	ReturnInteger(rdPtr->fillColor);
}

EXPRESSION(
	/* ID */			3,
	/* Name */			"LineColor(",
	/* Flags */			0,
	/* Params */		(0)
) {
	ReturnInteger(rdPtr->lineColor);
}

EXPRESSION(
	/* ID */			4,
	/* Name */			"LineWeight(",
	/* Flags */			0,
	/* Params */		(0)
) {
	ReturnInteger(rdPtr->lineWeight);
}

EXPRESSION(
	/* ID */			5,
	/* Name */			"FillColor2(",
	/* Flags */			0,
	/* Params */		(0)
) {
	ReturnInteger(rdPtr->fillColor2);
}

EXPRESSION(
	/* ID */			6,
	/* Name */			"Text$(",
	/* Flags */			0,
	/* Params */		(0)
) {
	ReturnStringSafe(rdPtr->text);
}
