// ============================================================================
//
// This file contains routines that are handled during the Runtime
// 
// ============================================================================

// Common Include
#include	"common.h"

#include <sstream>

void BuildPolySet (LPRDATA rdPtr) {
	if (rdPtr->polySet) {
		delete[] rdPtr->polySet;
	}

	// Recalculate number of coords
	rdPtr->polyn = 0;
	std::stringstream strc(rdPtr->polyCoords);
	while (strc.good()) {
		double v;
		while (strc.peek() == ',' || strc.peek() == ' ') strc.ignore(1);
		strc >> v;
		rdPtr->polyn++;
	}
	rdPtr->polyn /= 2;

	// Parse coords
	std::stringstream str(rdPtr->polyCoords);
	rdPtr->polySet = new tagPOINT[rdPtr->polyn];

	int preAdjust = (rdPtr->lineWeight - 1) / 2;
	int postAdjust = rdPtr->lineWeight;
	for (int i = 0; i < rdPtr->polyn; i++) {
		double a, b;
		while (str.peek() == ',' || str.peek() == ' ') str.ignore(1);
		str >> a;
		while (str.peek() == ',' || str.peek() == ' ') str.ignore(1);
		str >> b;
		rdPtr->polySet[i].x = (LONG)(preAdjust + a * (rdPtr->width - postAdjust + (rdPtr->width % 2)));
		rdPtr->polySet[i].y = (LONG)(preAdjust + b * (rdPtr->height - postAdjust + (rdPtr->height % 2)));
	}
}

void BuildSurface(LPRDATA rdPtr) {
	LPRH	rhPtr = rdPtr->rHo.hoAdRunHeader;
	mv*		mV = rdPtr->rHo.hoAdRunHeader->rh4.rh4Mv;

	LPSURFACE wSurf = WinGetSurface((int)mV->mvIdEditWin);

	if (wSurf != NULL) {
		int w = rdPtr->width;
		int h = rdPtr->height;

		cSurface* psf = NewSurface();
		psf->Create(w, h, wSurf);
		
		// Transparent color
		if (rdPtr->useTrans) {
			psf->Fill(rdPtr->transColor);
			psf->SetTransparentColor(rdPtr->transColor);
		}
		else {
			psf->Fill(RGB(0, 0, 0));
		}

		// Setup pattern if necessary
		CFillFlat* fdLine = 0;
		CFillData* fdFill = 0;
		cSurface* pattern = 0;
		switch (rdPtr->fillType) {
			case TFILL_GRAD:
				fdLine = new CFillFlat(rdPtr->lineColor);
				fdFill = (CFillData*) new CFillDir(rdPtr->fillColor, rdPtr->fillColor2, rdPtr->gradDir == GRAD_VERT);
				break;

			case TFILL_PATTERN:
				{
					cSurface base;
					LockImageSurface(mV->mvIdAppli, rdPtr->patterns[0], base, LOCKIMAGE_ALLREADACCESS);

					pattern = new cSurface();
					pattern->Create(base.GetWidth(), base.GetHeight(), wSurf);
					pattern->Fill(rdPtr->fillColor);
					base.Blit(*pattern, 0, 0, BMODE_TRANSP);
					UnlockImageSurface(base);

					fdLine = new CFillFlat(rdPtr->lineColor);
					fdFill = (CFillData*) new CFillMosaic(pattern);
				}
				break;

			case TFILL_IMAGE:
				{
					cSurface base;
					LockImageSurface(mV->mvIdAppli, rdPtr->images[0], base, LOCKIMAGE_ALLREADACCESS);

					cSurface imgbase;
					imgbase.Clone(base);

					DWORD opt = 0;
					BlitMode mode = BMODE_OPAQUE;
					if (rdPtr->imageOptions & IMGOPT_RESAMPLE && rdPtr->imageOptions & IMGOPT_RESAMPLE_EDGE)
						opt = STRF_RESAMPLE;
					else if (rdPtr->imageOptions & IMGOPT_RESAMPLE) {
						opt = STRF_RESAMPLE_TRANSP;
						mode = BMODE_TRANSP;
					}

					pattern = new cSurface();
					pattern->Create(rdPtr->width, rdPtr->height, wSurf);
					if (rdPtr->imageOptions & IMGOPT_RESAMPLE_EDGE)
						imgbase.ReplaceColor(rdPtr->fillColor, imgbase.GetTransparentColor());
					else
						pattern->Fill(rdPtr->fillColor);
					imgbase.Stretch(*pattern, 0, 0, rdPtr->width, rdPtr->height, mode, BOP_COPY, 0, opt);
					UnlockImageSurface(base);

					fdLine = new CFillFlat(rdPtr->lineColor);
					fdFill = (CFillData*) new CFillMosaic(pattern);
				}
				break;

		}

		// Rectangle Shape
		if (rdPtr->shape == TSHAPE_RECT) {
			if (rdPtr->fillType == TFILL_FLAT)
				psf->Rectangle(0, 0, w, h, rdPtr->fillColor, rdPtr->lineWeight, rdPtr->lineColor, 1); 
			else
				psf->Rectangle(0, 0, w, h, fdFill, rdPtr->lineWeight, fdLine, 0);
		}

		// Ellipse Shape
		else if (rdPtr->shape == TSHAPE_ELLIPSE) {
			if (rdPtr->fillType == TFILL_FLAT)
				psf->Ellipse(0, 0, w, h, rdPtr->fillColor, rdPtr->lineWeight, rdPtr->lineColor, 1);
			else
				psf->Ellipse(0, 0, w, h, fdFill, rdPtr->lineWeight, fdLine, 0);
		}

		// Polygon Shape
		else if (rdPtr->shape == TSHAPE_POLY) {
			if (rdPtr->fillType == TFILL_FLAT)
				psf->Polygon(rdPtr->polySet, rdPtr->polyn, rdPtr->fillColor, rdPtr->lineWeight, rdPtr->lineColor, 1);
			else
				psf->Polygon(rdPtr->polySet, rdPtr->polyn, fdFill, rdPtr->lineWeight, fdLine, 0);
		}

		// Text
		if (strlen(rdPtr->text) > 0) {
			HFONT hFnt = rdPtr->textFont; //CreateFontIndirect(&rdPtr->textFont);

			BlitOp bop = BOP_COPY;
			switch (rdPtr->textEffect & EFFECT_MASK) {
				case 1: bop = BOP_BLEND; break;
				case 2: bop = BOP_INVERT; break;
				case 3: bop = BOP_XOR; break;
				case 4: bop = BOP_AND; break;
				case 5: bop = BOP_OR; break;
				case 6: bop = BOP_MONO; break;
				case 7: bop = BOP_ADD; break;
				case 8: bop = BOP_SUB; break;
			}

			RECT rc;
			rc.left = 0; rc.top = 0;
			rc.right = rdPtr->width; rc.bottom = rdPtr->height;
			BOOL bAntiA = (rdPtr->textEffect & EFFECTFLAG_ANTIALIAS) ? TRUE : FALSE;
			//sf.TextOutA(ptrText(edPtr), edPtr->textLen, 0, 0, edPtr->textAlignFlags, &rc, edPtr->textColor, hFnt);
			DWORD dtFlags = rdPtr->textAlignFlags | DT_SINGLELINE | DT_NOPREFIX | DT_END_ELLIPSIS;
			psf->DrawTextA(rdPtr->text, strlen(rdPtr->text), &rc, dtFlags, rdPtr->textColor, hFnt, BMODE_TRANSP, bop, rdPtr->textCoef, bAntiA);
		}

		// Cleanup
		if (rdPtr->fillType == TFILL_GRAD) {
			delete fdFill;
			delete fdLine;
		}
		else if (rdPtr->fillType == TFILL_PATTERN) {
			delete fdFill;
			delete fdLine;
			delete pattern;
		}

		// Set Ink Effect & Etc.
		//BlitMode bm = (oiPtr->oiHdr.oiInkEffect & EFFECTFLAG_TRANSPARENT) ? BMODE_TRANSP : BMODE_OPAQUE;
		//BOOL bAntiA = (oiPtr->oiHdr.oiInkEffect & EFFECTFLAG_ANTIALIAS) ? TRUE : FALSE;
		//BlitOp bo = (BlitOp)(oiPtr->oiHdr.oiInkEffect & EFFECT_MASK);
		//int effectParam = oiPtr->oiHdr.oiInkEffectParam;

		//sf.Blit(*wSurf, x, y, bm, bo, effectParam, bAntiA ? BLTF_ANTIA : 0);

		if (rdPtr->pSf) {
			DeleteSurface(rdPtr->pSf);
		}

		if (rdPtr->pMask) {
			free(rdPtr->pMask);
			rdPtr->pMask = 0;
		}

		rdPtr->pSf = psf;
	}
}

void UpdateHotSpot (LPRDATA rdPtr) {
	int hx = rdPtr->hotX;
	int hy = rdPtr->hotY;

	if (rdPtr->rc.rcScaleX != 1)
		hx *= rdPtr->rc.rcScaleX;
	if (rdPtr->rc.rcScaleY != 1)
		hy *= rdPtr->rc.rcScaleY;

	if (rdPtr->rc.rcAngle != 0) {
		double rotx = cos(rdPtr->rc.rcAngle * PI / 180);
		double roty = sin(rdPtr->rc.rcAngle * PI / 180);

		int halfx = (rdPtr->width * rdPtr->rc.rcScaleX) / 2;
		int halfy = (rdPtr->height * rdPtr->rc.rcScaleY) / 2;
		int tx = ((hx - halfx) * rotx + (hy - halfy) * roty);
		int ty = ((hy - halfy) * rotx - (hx - halfx) * roty);

		hx = tx + rdPtr->transSf->GetWidth() / 2;
		hy = ty + rdPtr->transSf->GetHeight() / 2;
	}

	if (rdPtr->pMask) {
		rdPtr->pMask->mkXSpot = hx;
		rdPtr->pMask->mkYSpot = hy;
	}

	if (rdPtr->transMask) {
		rdPtr->transMask->mkXSpot = hx;
		rdPtr->transMask->mkYSpot = hy;
	}

	rdPtr->rHo.hoImgXSpot = hx;
	rdPtr->rHo.hoImgYSpot = hy;
}

void TransformSurface(LPRDATA rdPtr) {
	LPRH	rhPtr = rdPtr->rHo.hoAdRunHeader;
	mv*		mV = rdPtr->rHo.hoAdRunHeader->rh4.rh4Mv;

	// Cleanup previous transformed data
	if (rdPtr->transSf) {
		DeleteSurface(rdPtr->transSf);
		if (rdPtr->transMask) {
			free(rdPtr->transMask);
			rdPtr->transMask = 0;
		}
		rdPtr->transSf = 0;
		rdPtr->diffWidth = 0;
		rdPtr->diffHeight = 0;
	}

	if (rdPtr->rc.rcScaleX != 1 || rdPtr->rc.rcScaleY != 1) {
		int qscale = (rdPtr->rs.rsFlags & RSFLAG_SCALE_RESAMPLE) ? 7 : 0;
		rdPtr->transSf = NewSurface();
		rdPtr->transSf->Create(rdPtr->width * rdPtr->rc.rcScaleX, rdPtr->height * rdPtr->rc.rcScaleY, rdPtr->pSf);
		rdPtr->pSf->Stretch(*rdPtr->transSf, qscale);
	}

	if (rdPtr->rc.rcAngle != 0) {
		int qrot = (rdPtr->rs.rsFlags & RSFLAG_ROTATE_ANTIA) ? 7 : 0;
		if (rdPtr->transSf) {
			cSurface* temp = NewSurface();
			rdPtr->transSf->CreateRotatedSurface(*temp, (int) rdPtr->rc.rcAngle, qrot, rdPtr->transColor, 1);
			DeleteSurface(rdPtr->transSf);
			rdPtr->transSf = temp;
		}
		else {
			rdPtr->transSf = NewSurface();
			rdPtr->pSf->CreateRotatedSurface(*rdPtr->transSf, (int) rdPtr->rc.rcAngle, qrot, rdPtr->transColor, 1);
		}

		rdPtr->diffWidth = rdPtr->transSf->GetWidth() - (rdPtr->width * rdPtr->rc.rcScaleX);
		rdPtr->diffHeight = rdPtr->transSf->GetHeight() - (rdPtr->height * rdPtr->rc.rcScaleY);
	}

	// Transformed Mask
	if (rdPtr->transSf) {
		DWORD dwMaskSize = rdPtr->transSf->CreateMask(NULL, 0);
		if (dwMaskSize != 0)
		{
			rdPtr->transMask = (LPSMASK)calloc(dwMaskSize, 1);
			if (rdPtr->transMask != NULL)
			{
				rdPtr->transSf->CreateMask(rdPtr->transMask, 0);
			}
		}
	}

	rdPtr->rHo.hoImgWidth = (rdPtr->rc.rcScaleX * rdPtr->width) + rdPtr->diffWidth;
	rdPtr->rHo.hoImgHeight = (rdPtr->rc.rcScaleY * rdPtr->height) + rdPtr->diffHeight;
}

// --------------------
// GetRunObjectDataSize
// --------------------
// Returns the size of the runtime datazone of the object
// 
short WINAPI DLLExport GetRunObjectDataSize(fprh rhPtr, LPEDATA edPtr)
{
	return(sizeof(RUNDATA));
}


// ---------------
// CreateRunObject
// ---------------
// The routine where the object is actually created
// 
short WINAPI DLLExport CreateRunObject(LPRDATA rdPtr, LPEDATA edPtr, fpcob cobPtr)
{
	// Do some rSDK stuff
	#include "rCreateRunObject.h"
	
	/*
	   This routine runs when your object is created, as you might have guessed.
	   It is here that you must transfer any data you need in rdPtr from edPtr,
	   because after this has finished you cannot access it again!
	   Also, if you have anything to initialise (e.g. dynamic arrays, surface objects)
	   you should do it here, and free your resources in DestroyRunObject.
	   See Graphic_Object_Ex.txt for an example of what you may put here.
	*/

	rdPtr->rHo.hoX = cobPtr->cobX;
	rdPtr->rHo.hoY = cobPtr->cobY;
	rdPtr->rHo.hoImgWidth = edPtr->width;
	rdPtr->rHo.hoImgHeight = edPtr->height;

	//rdPtr->scaleX = 1.0;
	//rdPtr->scaleY = 1.0;
	//rdPtr->angle = 0;
	rdPtr->hotX = 0;
	rdPtr->hotY = 0;

	rdPtr->width = edPtr->width;
	rdPtr->height = edPtr->height;
	rdPtr->fillColor = edPtr->fillColor;
	rdPtr->fillColor2 = edPtr->fillColor2;
	rdPtr->gradDir = edPtr->gradDir;
	rdPtr->lineColor = edPtr->lineColor;
	rdPtr->lineWeight = edPtr->lineWeight;
	rdPtr->shape = edPtr->shape;
	rdPtr->fillType = edPtr->fillType;
	rdPtr->transColor = edPtr->transColor;
	rdPtr->useTrans = edPtr->useTrans;
	rdPtr->imageOptions = edPtr->imageOptions;

	rdPtr->patternCount = edPtr->patternCount;
	rdPtr->imageCount = edPtr->imageCount;

	rdPtr->patterns = new WORD[edPtr->patternCount];
	memcpy(rdPtr->patterns, ptrPatterns(edPtr), edPtr->patternCount * sizeof(WORD));
	
	rdPtr->images = new WORD[edPtr->imageCount];
	memcpy(rdPtr->images, ptrImages(edPtr), edPtr->imageCount * sizeof(WORD));

	rdPtr->textFont = CreateFontIndirect(&edPtr->textFont);
	rdPtr->textColor = edPtr->textColor;
	rdPtr->textAlignFlags = edPtr->textAlignFlags;
	rdPtr->textEffect = edPtr->textEffect;
	rdPtr->textCoef = edPtr->textCoef;

	rdPtr->text = new char[edPtr->textLen + 1];
	strncpy(rdPtr->text, ptrText(edPtr), edPtr->textLen + 1);

	rdPtr->polyn = edPtr->polyn;
	rdPtr->polySet = 0;
	
	rdPtr->polyCoords = new char[edPtr->polyCoordLen + 1];
	strncpy(rdPtr->polyCoords, ptrPolyCoords(edPtr), edPtr->polyCoordLen + 1);

	BuildPolySet(rdPtr);
	BuildSurface(rdPtr);

	rdPtr->transSf = 0;
	rdPtr->transMask = 0;
	rdPtr->rebuild = 0;
	rdPtr->rebuildPoly = 0;
	rdPtr->rebuildTrans = 0;

	// No errors
	return 0;
}


// ----------------
// DestroyRunObject
// ----------------
// Destroys the run-time object
// 
short WINAPI DLLExport DestroyRunObject(LPRDATA rdPtr, long fast)
{
/*
   When your object is destroyed (either with a Destroy action or at the end of
   the frame) this routine is called. You must free any resources you have allocated!
   See Graphic_Object_Ex.txt for an example of what you may put here.
*/
	delete[] rdPtr->patterns;
	delete[] rdPtr->images;
	delete[] rdPtr->text;
	delete[] rdPtr->polyCoords;

	if (rdPtr->textFont)
		DeleteObject(rdPtr->textFont);

	if (rdPtr->pSf)
		DeleteSurface(rdPtr->pSf);
	if (rdPtr->transSf)
		DeleteSurface(rdPtr->transSf);
	if (rdPtr->pMask)
		free(rdPtr->pMask);
	if (rdPtr->transMask)
		free(rdPtr->transMask);

	// No errors
	delete rdPtr->rRd;
	return 0;
}


// ----------------
// HandleRunObject
// ----------------
// Called (if you want) each loop, this routine makes the object live
// 
short WINAPI DLLExport HandleRunObject(LPRDATA rdPtr)
{
/*
   If your extension will draw to the MMF window you should first 
   check if anything about its display has changed :

       return rdPtr->roc.rcChanged?REFLAG_DISPLAY:0;

   You will also need to make sure you change this flag yourself 
   to 1 whenever you want to redraw your object
 
   If your extension won't draw to the window, but it still needs 
   to do something every MMF loop use :

        return 0;

   If you don't need to do something every loop, use :

        return REFLAG_ONESHOT;

   This doesn't mean this function can never run again. If you want MMF
   to handle your object again (causing this code to run) use this function:

        rdPtr->rRd->Rehandle();

   At the end of the loop this code will run
*/
	if (rdPtr->rc.rcChanged == 1) {
		rdPtr->rebuildTrans = 1;
	}

	if (rdPtr->rebuildPoly) {
		BuildPolySet(rdPtr);
		rdPtr->rebuildPoly = 0;
		if (rdPtr->shape == TSHAPE_POLY) {
			rdPtr->rebuild = 1;
		}
	}

	if (rdPtr->rebuild) {
		rdPtr->rHo.hoImgWidth = rdPtr->width;
		rdPtr->rHo.hoImgHeight = rdPtr->height;

		BuildSurface(rdPtr);
		rdPtr->rebuild = 0;
		rdPtr->rc.rcChanged = 1;
	}

	if (rdPtr->rebuildTrans) {
		TransformSurface(rdPtr);

		//npWin win = rdPtr->rHo.hoAdRunHeader->rh4.rh4Mv->mvIdEditWin;
		//SetSpriteAngle(win, rdPtr->rc.rcSprite, rdPtr->angle, 1);
		//ModifSpriteEx(win, rdPtr->rc.rcSprite, rdPtr->rc.rcSprite->sprX, rdPtr->rc.rcSprite->sprY, 0, 1, 1, 0, rdPtr->angle, 1);
		
		rdPtr->rebuildTrans = 0;
		rdPtr->rc.rcChanged = 1;
	}

	UpdateHotSpot(rdPtr);

	if (rdPtr->rc.rcChanged) {
		return REFLAG_DISPLAY;
	}

	return 0;
}

// ----------------
// DisplayRunObject
// ----------------
// Draw the object in the application screen.
// 
short WINAPI DLLExport DisplayRunObject(LPRDATA rdPtr)
{
/*
   If you return REFLAG_DISPLAY in HandleRunObject this routine will run.
   See Graphic_Object_Ex.txt for an example of what you may put here.
*/
	/*RECT rc;
	rc.left = rdPtr->rHo.hoX - rdPtr->rHo.hoAdRunHeader->rhWindowX;
	rc.top = rdPtr->rHo.hoY - rdPtr->rHo.hoAdRunHeader->rhWindowY;
	rc.right = rc.left + rdPtr->gridWidth;
	rc.bottom = rc.top + rdPtr->gridHeight;

	WinAddZone(rdPtr->rHo.hoAdRunHeader->rhIdEditWin, &rc);*/

	// Ok
	return 0;
}

// -------------------
// GetRunObjectSurface
// -------------------
// Implement this function instead of DisplayRunObject if your extension
// supports ink effects and transitions. Note: you can support ink effects
// in DisplayRunObject too, but this is automatically done if you implement
// GetRunObjectSurface (MMF applies the ink effect to the transition).
//
// Note: do not forget to enable the function in the .def file 
// if you remove the comments below.

cSurface* WINAPI DLLExport GetRunObjectSurface(LPRDATA rdPtr)
{
	/*RECT rc;
	rc.left = rdPtr->rHo.hoX - rdPtr->rHo.hoAdRunHeader->rhWindowX;
	rc.top = rdPtr->rHo.hoY - rdPtr->rHo.hoAdRunHeader->rhWindowY;
	rc.right = rc.left + rdPtr->rHo.hoImgWidth;
	rc.bottom = rc.top + rdPtr->rHo.hoImgHeight;*/

	WinAddZone(rdPtr->rHo.hoAdRunHeader->rhIdEditWin, &rdPtr->rHo.hoRect);

	if (rdPtr->transSf)
		return rdPtr->transSf;

	return rdPtr->pSf;
}


// -------------------------
// GetRunObjectCollisionMask
// -------------------------
// Implement this function if your extension supports fine collision mode (OEPREFS_FINECOLLISIONS),
// Or if it's a background object and you want Obstacle properties for this object.
//
// Should return NULL if the object is not transparent.
//
// Note: do not forget to enable the function in the .def file 
// if you remove the comments below.
//

sMask* WINAPI DLLExport GetRunObjectCollisionMask(LPRDATA rdPtr, LPARAM lParam)
{
	// Typical example for active objects
	// ----------------------------------
	// Opaque? collide with box
	if ( (rdPtr->rs.rsEffect & EFFECTFLAG_TRANSPARENT) == 0 )	// Note: only if your object has the OEPREFS_INKEFFECTS option
		return NULL;

	// Transparent? Create mask
	LPSMASK pMask = rdPtr->pMask;
	if ( pMask == NULL )
	{
		if ( rdPtr->pSf != NULL )
		{
			DWORD dwMaskSize = rdPtr->pSf->CreateMask(NULL, lParam);
			if ( dwMaskSize != 0 )
			{
				pMask = (LPSMASK)calloc(dwMaskSize, 1);
				if ( pMask != NULL )
				{
					rdPtr->pSf->CreateMask(pMask, lParam);
					rdPtr->pMask = pMask;
					pMask->mkXSpot = rdPtr->rHo.hoImgXSpot;
					pMask->mkYSpot = rdPtr->rHo.hoImgYSpot;
				}
			}
		}
	}

	// Note: for active objects, lParam is always the same.
	// For background objects (OEFLAG_BACKGROUND), lParam maybe be different if the user uses your object
	// as obstacle and as platform. In this case, you should store 2 collision masks
	// in your data: one if lParam is 0 and another one if lParam is different from 0.

	if (rdPtr->transMask)
		return rdPtr->transMask;

	return pMask;
}


// ----------------
// PauseRunObject
// ----------------
// Enters the pause mode
// 
short WINAPI DLLExport PauseRunObject(LPRDATA rdPtr)
{
	// Ok
	return 0;
}


// -----------------
// ContinueRunObject
// -----------------
// Quits the pause mode
//
short WINAPI DLLExport ContinueRunObject(LPRDATA rdPtr)
{
	// Ok
	return 0;
}


// ============================================================================
//
// START APP / END APP / START FRAME / END FRAME routines
// 
// ============================================================================

// -------------------
// StartApp
// -------------------
// Called when the application starts or restarts.
// Useful for storing global data
// 
void WINAPI DLLExport StartApp(mv _far *mV, CRunApp* pApp)
{
	// Example
	// -------
	// Delete global data (if restarts application)
//	CMyData* pData = (CMyData*)mV->mvGetExtUserData(pApp, hInstLib);
//	if ( pData != NULL )
//	{
//		delete pData;
//		mV->mvSetExtUserData(pApp, hInstLib, NULL);
//	}
}

// -------------------
// EndApp
// -------------------
// Called when the application ends.
// 
void WINAPI DLLExport EndApp(mv _far *mV, CRunApp* pApp)
{
	// Example
	// -------
	// Delete global data
//	CMyData* pData = (CMyData*)mV->mvGetExtUserData(pApp, hInstLib);
//	if ( pData != NULL )
//	{
//		delete pData;
//		mV->mvSetExtUserData(pApp, hInstLib, NULL);
//	}
}

// -------------------
// StartFrame
// -------------------
// Called when the frame starts or restarts.
// 
void WINAPI DLLExport StartFrame(mv _far *mV, DWORD dwReserved, int nFrameIndex)
{

}

// -------------------
// EndFrame
// -------------------
// Called when the frame ends.
// 
void WINAPI DLLExport EndFrame(mv _far *mV, DWORD dwReserved, int nFrameIndex)
{

}

// ============================================================================
//
// TEXT ROUTINES (if OEFLAG_TEXT)
// 
// ============================================================================

// -------------------
// GetRunObjectFont
// -------------------
// Return the font used by the object.
// 


  // Note: do not forget to enable the functions in the .def file 
  // if you remove the comments below.

void WINAPI GetRunObjectFont(LPRDATA rdPtr, LOGFONT* pLf)
{
	// Example
	// -------
	GetObject(rdPtr->textFont, sizeof(LOGFONT), pLf);
}

// -------------------
// SetRunObjectFont
// -------------------
// Change the font used by the object.
// 
void WINAPI SetRunObjectFont(LPRDATA rdPtr, LOGFONT* pLf, RECT* pRc)
{
	// Example
	// -------
	HFONT hFont = CreateFontIndirect(pLf);
	if ( hFont != NULL )
	{
		if (rdPtr->textFont != 0)
			DeleteObject(rdPtr->textFont);
		rdPtr->textFont = hFont;
		//SendMessage(rdPtr->m_hWnd, WM_SETFONT, (WPARAM)rdPtr->m_hFont, FALSE);
		rdPtr->rebuild = 1;
		callRunTimeFunction(rdPtr, RFUNCTION_REDRAW, 0, 0);
	}

}

// ---------------------
// GetRunObjectTextColor
// ---------------------
// Return the text color of the object.
// 
COLORREF WINAPI GetRunObjectTextColor(LPRDATA rdPtr)
{
	// Example
	// -------
	return rdPtr->textColor;
}

// ---------------------
// SetRunObjectTextColor
// ---------------------
// Change the text color of the object.
// 
void WINAPI SetRunObjectTextColor(LPRDATA rdPtr, COLORREF rgb)
{
	// Example
	// -------
	rdPtr->textColor = rgb;
	//InvalidateRect(rdPtr->m_hWnd, NULL, TRUE);
	rdPtr->rebuild = 1;
	callRunTimeFunction(rdPtr, RFUNCTION_REDRAW, 0, 0);
}



// ============================================================================
//
// DEBUGGER ROUTINES
// 
// ============================================================================

// -----------------
// GetDebugTree
// -----------------
// This routine returns the address of the debugger tree
//
LPWORD WINAPI DLLExport GetDebugTree(LPRDATA rdPtr)
{
#if !defined(RUN_ONLY)
	return DebugTree;
#else
	return NULL;
#endif // !defined(RUN_ONLY)
}

// -----------------
// GetDebugItem
// -----------------
// This routine returns the text of a given item.
//
void WINAPI DLLExport GetDebugItem(LPSTR pBuffer, LPRDATA rdPtr, int id)
{
#if !defined(RUN_ONLY)

	// Example
	// -------
/*
	char temp[DB_BUFFERSIZE];

	switch (id)
	{
	case DB_CURRENTSTRING:
		LoadString(hInstLib, IDS_CURRENTSTRING, temp, DB_BUFFERSIZE);
		wsprintf(pBuffer, temp, rdPtr->text);
		break;
	case DB_CURRENTVALUE:
		LoadString(hInstLib, IDS_CURRENTVALUE, temp, DB_BUFFERSIZE);
		wsprintf(pBuffer, temp, rdPtr->value);
		break;
	case DB_CURRENTCHECK:
		LoadString(hInstLib, IDS_CURRENTCHECK, temp, DB_BUFFERSIZE);
		if (rdPtr->check)
			wsprintf(pBuffer, temp, "TRUE");
		else
			wsprintf(pBuffer, temp, "FALSE");
		break;
	case DB_CURRENTCOMBO:
		LoadString(hInstLib, IDS_CURRENTCOMBO, temp, DB_BUFFERSIZE);
		wsprintf(pBuffer, temp, rdPtr->combo);
		break;
	}
*/

#endif // !defined(RUN_ONLY)
}

// -----------------
// EditDebugItem
// -----------------
// This routine allows to edit editable items.
//
void WINAPI DLLExport EditDebugItem(LPRDATA rdPtr, int id)
{
#if !defined(RUN_ONLY)

	// Example
	// -------
/*
	switch (id)
	{
	case DB_CURRENTSTRING:
		{
			EditDebugInfo dbi;
			char buffer[256];

			dbi.pText=buffer;
			dbi.lText=TEXT_MAX;
			dbi.pTitle=NULL;

			strcpy(buffer, rdPtr->text);
			long ret=callRunTimeFunction(rdPtr, RFUNCTION_EDITTEXT, 0, (LPARAM)&dbi);
			if (ret)
				strcpy(rdPtr->text, dbi.pText);
		}
		break;
	case DB_CURRENTVALUE:
		{
			EditDebugInfo dbi;

			dbi.value=rdPtr->value;
			dbi.pTitle=NULL;

			long ret=callRunTimeFunction(rdPtr, RFUNCTION_EDITINT, 0, (LPARAM)&dbi);
			if (ret)
				rdPtr->value=dbi.value;
		}
		break;
	}
*/
#endif // !defined(RUN_ONLY)
}


